/* This file is generated by fsm-compiler; do not edit */

/* -*-C-*- */

/*
 *  Copyright (C) 2006, 2007 Greg Benison
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include <string.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>
#include "contour-fsm.h"

/* #define FSM_VERBOSE */

/* Traps exist only as convenient hooks for the debugger. */
static void trap_stray() {}
static void trap_dump() {}
static void trap_hit_mark() {}

#ifdef ENABLE_TRAPS
#define TRAP(op) {op();}
#else
#define TRAP(op)
#endif /* ENABLE_TRAPS */

void
test_method(void* data, struct hos_point* points, const int n_point, const int level, gboolean closed)
{
  char* msg = closed ? "closed" : "open";
  printf("level %d (%s):\n", level, msg);

  struct hos_point* cur;
  for (cur = points; cur < points + n_point; ++cur)
    printf(" %f, %f\n", cur->x, cur->y);

}

struct _fsm_state
{
  /* buffer = nx * ny array of data */
  double *buffer;
  int nx;
  int ny;

  /* The rectangle to search for contours, and the current row */
  int x_first;
  int x_last;
  int y_first;
  int y_last;
  int row;

  /* The contour levels */
  double *levels;
  int n_levels;

  /* The marks buffer, for which contours have already been traced */
  char* marks;
  int mark_stride;

  trace_func_t trace_method;
  gpointer trace_data;
};

fsm_state_t*
fsm_state_init(double* buffer,
	       int nx,
	       int ny,
	       int x_first,
	       int x_last,
	       int y_first,
	       int y_last,
	       double* levels,
	       int n_levels,
	       trace_func_t trace_method,
	       gpointer trace_data)
{
  assert(buffer != NULL);
  fsm_state_t* result = g_new0(fsm_state_t, 1);

  assert (buffer != NULL);

  /* bounds checking */
  x_first = CLAMP(x_first, 0, nx - 1);
  x_last  = CLAMP(x_last,  0, nx - 1);
  y_first = CLAMP(y_first, 0, ny - 1);
  y_last  = CLAMP(y_last,  0, ny - 1);

  result->buffer = buffer;
  result->nx = nx;
  result->ny = ny;
  result->x_first = x_first;
  result->x_last = x_last;
  result->y_first = y_first;
  result->y_last = y_last;
  result->levels = levels;
  result->n_levels = n_levels;
  result->trace_method = trace_method;
  result->trace_data = trace_data;

  result->row = y_first;

  result->mark_stride = (n_levels / 8) + 1;
  int mark_nx = (x_last - x_first + 1);
  int mark_ny = (y_last - y_first + 1);
  result->marks = g_new0(char, mark_nx * mark_ny * result->mark_stride);

  return result;
}

void
fsm_state_free(fsm_state_t* state)
{
  g_free(state->marks);
  g_free(state);
}

/*
 * In the 2D array 'buffer', of size nx * ny,
 * trace all contours emanating from row 'row',
 * within columns x_first --> x_last.
 *
 * Trace contours for all thresholds in the array 'levels'
 * (of length 'n_levels'), calling
 * 'trace_method' with extra data 'trace_data'
 * for every contour loop found.
 * 
 * The mark array is a bitvector used in the contour tracing process.
 * There is one bit, for every contour level, for every edge in the
 * spectrum.  A one-to-one correspondence can be made between points in
 * the spectrum and edges.  If a contour line is discovered crossing an
 * edge, the bit corresponding to that edge is set.  Each point in the
 * spectrum is allocated 'mark_stride' number of bytes; there must be
 * enough bits to accomodate all the contour levels.
 *
 * returns: finished tracing this 'state' ?
 */
gboolean
contour_fsm(fsm_state_t* state)
{

#define buffer        state->buffer       
#define nx	      state->nx           
#define ny	      state->ny           
#define x_first	      state->x_first      
#define x_last	      state->x_last       
#define y_first	      state->y_first      
#define y_last	      state->y_last       
#define row	      state->row          
#define levels	      state->levels       
#define n_levels      state->n_levels     
#define marks	      state->marks        
#define mark_stride   state->mark_stride  
#define trace_method  state->trace_method 
#define trace_data    state->trace_data   

  assert(buffer != NULL);

  if (row > y_last)
    return TRUE;

#ifdef FSM_VERBOSE
fprintf(stderr, "- fsm start tracing row %d -\n", row);
#endif	

  /******** level pointer ********/
  enum {lvl_range_below, lvl_range_mid, lvl_range_above};
  gint lvl_range;
  gint lvl_idx;
#define CURRENT_LEVEL levels[lvl_idx]
#define LVL_INCREMENT {							\
    {if (lvl_range == lvl_range_below) { lvl_idx = 0; lvl_range = lvl_range_mid; } \
    else if (lvl_range == lvl_range_mid) { ++lvl_idx;			\
      if (lvl_idx >= n_levels) lvl_range = lvl_range_above;}} REPORT_LVL; }

#define LVL_DECREMENT { \
    {if (lvl_range == lvl_range_above) { lvl_idx = (n_levels - 1); lvl_range = lvl_range_mid; } \
    else if (lvl_range == lvl_range_mid) { lvl_idx--;			\
      if (lvl_idx < 0) lvl_range = lvl_range_below;}} REPORT_LVL; }

#define REPORT_LVL
#ifdef FSM_VERBOSE
#define REPORT_LVL { fprintf(stderr, " - level "); \
    if (lvl_range == lvl_range_above) fprintf(stderr, " = top -\n"); \
    if (lvl_range == lvl_range_below) fprintf(stderr, " = bottom -\n"); \
    if (lvl_range == lvl_range_mid)   fprintf(stderr, " = %d -\n", lvl_idx); }
#endif

#define LVL_SET_ABOVE  {lvl_range = lvl_range_above;}
#define LVL_SET_BELOW  {lvl_range = lvl_range_below;}
#define LVL_OVERFLOW_P  (lvl_range == lvl_range_above)
#define LVL_UNDERFLOW_P  (lvl_range == lvl_range_below)
#define LVL_ABOVE_P(_lvl_) (LVL_OVERFLOW_P ? 1 : (LVL_UNDERFLOW_P ? 0 : (CURRENT_LEVEL > _lvl_)))
#define LVL_BELOW_P(_lvl_) (LVL_OVERFLOW_P ? 0 : (LVL_UNDERFLOW_P ? 1 : (CURRENT_LEVEL <= _lvl_)))

  /******** the 'search edge' **********/
  double *edge_this = NULL, *edge_next = NULL;

#define START_PT (buffer + x_first + (nx * row))
#define LAST_PT (buffer + x_last + (nx * row))
#define START_PT_NORTH (START_PT + nx)
#define START_PT_EAST (START_PT + 1)

  /******** marks buffer **********/

  int mark_x, mark_y;
  char* mark_ptr;

#define MARK_BUMP_EAST {mark_x++ ; MARK_SET_PTR;}
#define MARK_BUMP_WEST {mark_x-- ; MARK_SET_PTR;}
#define MARK_BUMP_NORTH {mark_y++; MARK_SET_PTR;}
#define MARK_BUMP_SOUTH {mark_y--; MARK_SET_PTR;}

#define MARK_INIT_COORDS(_x_, _y_) {mark_x = _x_; mark_y = _y_; MARK_SET_PTR;}
#define MARK_IN_REGION_P ( (mark_x >= x_first) && \
			   (mark_x <= x_last)  && \
			   (mark_y >= y_first) && \
			   (mark_y <= y_last))
#define MARK_SET_PTR { mark_ptr = MARK_IN_REGION_P ? marks + MARK_PTR_OFFSET : NULL; }
#define MARK_PTR_OFFSET   (((mark_x - x_first) + \
			    (mark_y - y_first) * (x_last - x_first + 1)) * mark_stride)

#define MARK_SET_P   (mark_ptr ? (*(mark_ptr + (lvl_idx / 8)) & (1 << (lvl_idx % 8))) : FALSE)
#define MARK_SET     { if (mark_ptr) *(mark_ptr + (lvl_idx / 8)) |= (1 << (lvl_idx % 8)); }

#define HIT_MARK_TRAP {TRAP(trap_hit_mark);}

  /******** results buffer **********/

  GArray* result_buffer = g_array_new(FALSE, FALSE, sizeof(struct hos_point));

#define RESULT_DATA ((struct hos_point*)(result_buffer->data))
#define RESULT_LENGTH (result_buffer->len)

#define RESULT_X(_idx_) g_array_index(result_buffer, struct hos_point, (_idx_)).x
#define RESULT_Y(_idx_) g_array_index(result_buffer, struct hos_point, (_idx_)).y

#define RESULT_FLIP { int i; struct hos_point tmp; \
    for (i = 0; (double)i < (result_buffer->len / 2.0); ++i) {	\
      tmp.x = RESULT_X(i);					\
      tmp.y = RESULT_Y(i);					\
      RESULT_X(i) = RESULT_X(result_buffer->len - i - 1);		\
      RESULT_Y(i) = RESULT_Y(result_buffer->len - i - 1);		\
      RESULT_X(result_buffer->len - i - 1) = tmp.x;			\
      RESULT_Y(result_buffer->len - i - 1) = tmp.y; }}
#define RESULT_RESET { g_array_set_size(result_buffer, 0); }
#define RESULT_PUSH(_x_, _y_)  { struct hos_point new_value = {(_x_), (_y_)}; \
                                 g_array_append_val(result_buffer, new_value); }
#define RESULT_POP { g_array_set_size(result_buffer, result_buffer->len - 1); }
#define RESULT_FREE { g_array_free(result_buffer, TRUE); result_buffer = NULL; }

  /******** contour tracing **********/

#define CONTOUR_DRAW_CLOSED {trace_method(trace_data, RESULT_DATA, result_buffer->len, lvl_idx, TRUE);}
#define CONTOUR_DRAW_OPEN {trace_method(trace_data, RESULT_DATA, result_buffer->len, lvl_idx, FALSE);}

  /******** contour searching *********/

  /* registers */
  double* cntr_ptr;
  int cntr_x, cntr_y;
  int cntr_start_x, cntr_start_y;

#define CONTOUR_AT_START_P ((cntr_x == cntr_start_x) && (cntr_y == cntr_start_y) && (RESULT_LENGTH > 1))
#define CONTOUR_INIT_COORDS(_x_, _y_)  {cntr_x = _x_; cntr_y = _y_; cntr_ptr = buffer + _x_ + (_y_ * nx); \
                                        cntr_start_x = cntr_x; cntr_start_y = cntr_y; \
                                        MARK_INIT_COORDS(_x_, _y_);}

#define CONTOUR_GOTO_START    { CONTOUR_INIT_COORDS(cntr_start_x, cntr_start_y) }

#define CONTOUR_EAST_EDGE_P   (cntr_x >= (nx - 1))
#define CONTOUR_WEST_EDGE_P   (cntr_x <= 0)
#define CONTOUR_NORTH_EDGE_P  (cntr_y >= (ny - 1))
#define CONTOUR_SOUTH_EDGE_P  (cntr_y <= 0)

#define CONTOUR_STRADDLES_P(_p1_, _p2_) ((_p1_ > _p2_) ? \
	                      STRADDLES_INNER(_p2_, _p1_) : STRADDLES_INNER(_p1_, _p2_))
#define STRADDLES_INNER(_lo_, _hi_)  (LVL_BELOW_P(_hi_) && LVL_ABOVE_P(_lo_))

#define CONTOUR_PT_CC *cntr_ptr
#define CONTOUR_PT_NC *(cntr_ptr + nx)
#define CONTOUR_PT_SC *(cntr_ptr - nx)

#define CONTOUR_PT_CW *(cntr_ptr - 1)
#define CONTOUR_PT_NW *(cntr_ptr - 1 + nx)
#define CONTOUR_PT_SW *(cntr_ptr - 1 - nx)

#define CONTOUR_PT_CE *(cntr_ptr + 1)
#define CONTOUR_PT_NE *(cntr_ptr + 1 + nx)
#define CONTOUR_PT_SE *(cntr_ptr + 1 - nx)

#define CONTOUR_BUMP_EAST  {++cntr_ptr; ++cntr_x; MARK_BUMP_EAST;}
#define CONTOUR_BUMP_WEST  {--cntr_ptr; --cntr_x; MARK_BUMP_WEST;}
#define CONTOUR_BUMP_NORTH  {cntr_ptr += nx; ++cntr_y; MARK_BUMP_NORTH;}
#define CONTOUR_BUMP_SOUTH {cntr_ptr -= nx; --cntr_y; MARK_BUMP_SOUTH;}

  /* interpolation */
#define INTERPOLATE(_pt1_, _pt2_) ((_pt2_ < _pt1_) ? \
                (1.0 - INTERPOLATE_INNER(_pt2_, _pt1_)) : INTERPOLATE_INNER(_pt1_, _pt2_))
#define INTERPOLATE_INNER(_ipt1_, _ipt2_) ((CURRENT_LEVEL - _ipt1_) / (_ipt2_ - _ipt1_))

#define CONTOUR_PAINT_HORIZONTAL {RESULT_PUSH(cntr_x + INTERPOLATE(CONTOUR_PT_CC, CONTOUR_PT_CE), cntr_y);}
#define CONTOUR_PAINT_VERTICAL   {RESULT_PUSH(cntr_x, cntr_y + INTERPOLATE(CONTOUR_PT_CC, CONTOUR_PT_NC));}

#define ABOVE_STR(x)((x > CURRENT_LEVEL) ? "*" : " ")

#define CONTOUR_DUMP {TRAP(trap_dump); \
                      fprintf(stderr, "contour coordinates (lvl %f)\n", level); \
                      int i; \
                      for (i = 0; i < RESULT_LENGTH; ++i) { \
                          double result_x = ((g_array_index(result_buffer, struct hos_point, i)).x); \
                          double result_y = ((g_array_index(result_buffer, struct hos_point, i)).y); \
                          int int_result_x = floor(result_x); \
                          int int_result_y = floor(result_y); \
                          double vl_ll = *(buffer + int_result_x + int_result_y * nx); \
                          double vl_ul = *(buffer + int_result_x + 1 + int_result_y * nx); \
                          double vl_lu = *(buffer + int_result_x + (int_result_y + 1) * nx); \
                          double vl_uu = *(buffer + int_result_x + 1 + (int_result_y + 1) * nx); \
                          fprintf(stderr, "%.2f %.2f\n", result_x, result_y); \
                      }}

  /******* error recovery **********/
#define FSM_TRAPPED { fprintf(stderr, "fsm: trapped in state %s\n", fsm_current_state); \
                      goto contour_done;  }

  /******* actions & conditions *******/

#define WEST_EDGE_SEARCH_INIT__CONDITION  (row < (ny - 1))
#define WEST_EDGE_SEARCH_INIT__ACTION     {edge_this = START_PT; edge_next = START_PT_NORTH;}

#define LVL_SEEK_FROM_TOP__CONDITION    (*edge_next <= *edge_this)
#define LVL_SEEK_FROM_BOTTOM__CONDITION (*edge_next > *edge_this)
#define LVL_SEEK_FROM_TOP__ACTION       {LVL_SET_ABOVE; while (LVL_ABOVE_P(*edge_this)) LVL_DECREMENT;}
#define LVL_SEEK_FROM_BOTTOM__ACTION    {LVL_SET_BELOW; while (LVL_BELOW_P(*edge_this)) LVL_INCREMENT;}

#define LVL_ASCEND_COMPLETE__CONDITION  (LVL_ABOVE_P(*edge_next))
#define LVL_DESCEND_COMPLETE__CONDITION (LVL_BELOW_P(*edge_next))
#define LVL_INCREMENT__ACTION           {LVL_INCREMENT}
#define LVL_DECREMENT__ACTION           {LVL_DECREMENT}

#define HORIZONTAL_SEARCH_INIT__ACTION  {edge_this = START_PT; edge_next = START_PT_EAST;}
#define COLUMN_INCREASE__ACTION         {++edge_this; ++edge_next;}
#define HORIZONTAL_SEARCH_COMPLETE__CONDITION (edge_next > LAST_PT)
#define HORIZONTAL_SEARCH_COMPLETE__ACTION { goto contour_done; }

#define CONTINUE_UPWARDS__CONDITION   (*edge_next >= *edge_this)
#define CONTINUE_DOWNWARDS__CONDITION (*edge_next < *edge_this)
#define TURN_UPWARDS__CONDITION       (*edge_next >= *edge_this)
#define TURN_DOWNWARDS__CONDITION     (*edge_next < *edge_this)
#define TURN_UPWARDS__ACTION          {LVL_INCREMENT;}
#define TURN_DOWNWARDS__ACTION         {LVL_DECREMENT;}

#define END_FIRST_LEG__ACTION           { RESULT_FLIP; RESULT_POP; CONTOUR_GOTO_START; }

#define TRACE_START__ACTION             { RESULT_RESET; CONTOUR_INIT_COORDS((edge_this - buffer) % nx, row); }
#define TRACE_DRAW_CLOSED__ACTION       CONTOUR_DRAW_CLOSED
#define TRACE_DRAW_OPEN__ACTION         CONTOUR_DRAW_OPEN
#define TRACE_FOUND_LOOP__CONDITION     CONTOUR_AT_START_P

#define AT_SOUTH_EDGE__CONDITION        CONTOUR_SOUTH_EDGE_P
#define AT_NORTH_EDGE__CONDITION        CONTOUR_NORTH_EDGE_P
#define AT_EAST_EDGE__CONDITION         CONTOUR_EAST_EDGE_P
#define AT_WEST_EDGE__CONDITION         CONTOUR_WEST_EDGE_P

#define GO_NORTH_TOWARD_NORTH__CONDITION   ( CONTOUR_STRADDLES_P(CONTOUR_PT_NC, CONTOUR_PT_NE) )
#define GO_NORTH_TOWARD_EAST__CONDITION    ( CONTOUR_STRADDLES_P(CONTOUR_PT_CE, CONTOUR_PT_NE) )
#define GO_NORTH_TOWARD_WEST__CONDITION    ( CONTOUR_STRADDLES_P(CONTOUR_PT_CC, CONTOUR_PT_NC) )

#define GO_SOUTH_TOWARD_SOUTH__CONDITION   ( CONTOUR_STRADDLES_P(CONTOUR_PT_SC, CONTOUR_PT_SE) )
#define GO_SOUTH_TOWARD_EAST__CONDITION    ( CONTOUR_STRADDLES_P(CONTOUR_PT_CE, CONTOUR_PT_SE) )
#define GO_SOUTH_TOWARD_WEST__CONDITION    ( CONTOUR_STRADDLES_P(CONTOUR_PT_CC, CONTOUR_PT_SC) )

#define GO_EAST_TOWARD_NORTH__CONDITION    ( CONTOUR_STRADDLES_P(CONTOUR_PT_NC, CONTOUR_PT_NE) )
#define GO_EAST_TOWARD_SOUTH__CONDITION    ( CONTOUR_STRADDLES_P(CONTOUR_PT_CC, CONTOUR_PT_CE) )
#define GO_EAST_TOWARD_EAST__CONDITION	    ( CONTOUR_STRADDLES_P(CONTOUR_PT_CE, CONTOUR_PT_NE) )

#define GO_WEST_TOWARD_NORTH__CONDITION    ( CONTOUR_STRADDLES_P(CONTOUR_PT_NW, CONTOUR_PT_NC) )
#define GO_WEST_TOWARD_SOUTH__CONDITION    ( CONTOUR_STRADDLES_P(CONTOUR_PT_CW, CONTOUR_PT_CC) )
#define GO_WEST_TOWARD_WEST__CONDITION     ( CONTOUR_STRADDLES_P(CONTOUR_PT_CW, CONTOUR_PT_NW) )

#define GO_NORTH_TOWARD_NORTH__ACTION   { CONTOUR_BUMP_NORTH; }
#define GO_NORTH_TOWARD_EAST__ACTION    { CONTOUR_BUMP_EAST; }
#define GO_NORTH_TOWARD_WEST__ACTION    { /* no-op */ }

#define GO_SOUTH_TOWARD_SOUTH__ACTION   { CONTOUR_BUMP_SOUTH; }
#define GO_SOUTH_TOWARD_EAST__ACTION    { CONTOUR_BUMP_SOUTH; CONTOUR_BUMP_EAST; }
#define GO_SOUTH_TOWARD_WEST__ACTION    { CONTOUR_BUMP_SOUTH; }

#define GO_EAST_TOWARD_NORTH__ACTION    { CONTOUR_BUMP_NORTH; }
#define GO_EAST_TOWARD_SOUTH__ACTION    { /* no-op */ }
#define GO_EAST_TOWARD_EAST__ACTION	 { CONTOUR_BUMP_EAST; }

#define GO_WEST_TOWARD_NORTH__ACTION    { CONTOUR_BUMP_WEST; CONTOUR_BUMP_NORTH; }
#define GO_WEST_TOWARD_SOUTH__ACTION    { CONTOUR_BUMP_WEST; }
#define GO_WEST_TOWARD_WEST__ACTION     { CONTOUR_BUMP_WEST; }

#define FACING_NORTH__ACTION { MARK_SET; CONTOUR_PAINT_HORIZONTAL; }
#define FACING_SOUTH__ACTION { MARK_SET; CONTOUR_PAINT_HORIZONTAL; }
#define FACING_WEST__ACTION   CONTOUR_PAINT_VERTICAL
#define FACING_EAST__ACTION   CONTOUR_PAINT_VERTICAL


#define TRACE_HIT_MARK__CONDITION       ((MARK_SET_P) && !(CONTOUR_AT_START_P))

static char* fsm_current_state;
S0:
fsm_current_state = "start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH_INIT__CONDITION
#define WEST_EDGE_SEARCH_INIT__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH_INIT__ACTION
#define WEST_EDGE_SEARCH_INIT__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH_INIT__CONDITION))) { WEST_EDGE_SEARCH_INIT__ACTION; goto S1;}
#ifndef HORIZONTAL_SEARCH_INIT__CONDITION
#define HORIZONTAL_SEARCH_INIT__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_INIT__ACTION
#define HORIZONTAL_SEARCH_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_INIT__CONDITION))) { HORIZONTAL_SEARCH_INIT__ACTION; goto S2;}
goto S_TRAPPED;
/*********/
S2:
fsm_current_state = "horizontal-search-init";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_SEEK_INIT__CONDITION
#define LVL_SEEK_INIT__CONDITION (1)
#endif
#ifndef LVL_SEEK_INIT__ACTION
#define LVL_SEEK_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_SEEK_INIT__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_SEEK_INIT__ACTION; goto S3;}
goto S_TRAPPED;
/*********/
S1:
fsm_current_state = "west-edge-search-init";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_SEEK_INIT__CONDITION
#define LVL_SEEK_INIT__CONDITION (1)
#endif
#ifndef LVL_SEEK_INIT__ACTION
#define LVL_SEEK_INIT__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (LVL_SEEK_INIT__CONDITION))) { WEST_EDGE_SEARCH__ACTION; LVL_SEEK_INIT__ACTION; goto S4;}
goto S_TRAPPED;
/*********/
S4:
fsm_current_state = "west-edge-search:lvl-seek-init";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEEK_FROM_TOP__CONDITION
#define LVL_SEEK_FROM_TOP__CONDITION (1)
#endif
#ifndef LVL_SEEK_FROM_TOP__ACTION
#define LVL_SEEK_FROM_TOP__ACTION { /* no-op */ }
#endif
if (((LVL_SEEK_FROM_TOP__CONDITION))) { LVL_SEEK_FROM_TOP__ACTION; goto S5;}
#ifndef LVL_SEEK_FROM_BOTTOM__CONDITION
#define LVL_SEEK_FROM_BOTTOM__CONDITION (1)
#endif
#ifndef LVL_SEEK_FROM_BOTTOM__ACTION
#define LVL_SEEK_FROM_BOTTOM__ACTION { /* no-op */ }
#endif
if (((LVL_SEEK_FROM_BOTTOM__CONDITION))) { LVL_SEEK_FROM_BOTTOM__ACTION; goto S6;}
goto S_TRAPPED;
/*********/
S3:
fsm_current_state = "horizontal-search:lvl-seek-init";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEEK_FROM_TOP__CONDITION
#define LVL_SEEK_FROM_TOP__CONDITION (1)
#endif
#ifndef LVL_SEEK_FROM_TOP__ACTION
#define LVL_SEEK_FROM_TOP__ACTION { /* no-op */ }
#endif
if (((LVL_SEEK_FROM_TOP__CONDITION))) { LVL_SEEK_FROM_TOP__ACTION; goto S7;}
#ifndef LVL_SEEK_FROM_BOTTOM__CONDITION
#define LVL_SEEK_FROM_BOTTOM__CONDITION (1)
#endif
#ifndef LVL_SEEK_FROM_BOTTOM__ACTION
#define LVL_SEEK_FROM_BOTTOM__ACTION { /* no-op */ }
#endif
if (((LVL_SEEK_FROM_BOTTOM__CONDITION))) { LVL_SEEK_FROM_BOTTOM__ACTION; goto S8;}
goto S_TRAPPED;
/*********/
S8:
fsm_current_state = "horizontal-search:lvl-seek-from-bottom";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S9;}
goto S_TRAPPED;
/*********/
S7:
fsm_current_state = "horizontal-search:lvl-seek-from-top";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S10;}
goto S_TRAPPED;
/*********/
S6:
fsm_current_state = "west-edge-search:lvl-seek-from-bottom";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S11;}
goto S_TRAPPED;
/*********/
S5:
fsm_current_state = "west-edge-search:lvl-seek-from-top";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S12;}
goto S_TRAPPED;
/*********/
S12:
fsm_current_state = "west-edge-search:lvl-search-descend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DESCEND_COMPLETE__CONDITION
#define LVL_DESCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_DESCEND_COMPLETE__ACTION
#define LVL_DESCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_DESCEND_COMPLETE__CONDITION))) { LVL_DESCEND_COMPLETE__ACTION; goto S13;}
#ifndef LVL_SEARCH_DESCENDING__CONDITION
#define LVL_SEARCH_DESCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCENDING__ACTION
#define LVL_SEARCH_DESCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_DESCENDING__ACTION; TRACE_START__ACTION; goto S14;}
goto S_TRAPPED;
/*********/
S11:
fsm_current_state = "west-edge-search:lvl-search-ascend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_ASCEND_COMPLETE__CONDITION
#define LVL_ASCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_ASCEND_COMPLETE__ACTION
#define LVL_ASCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_ASCEND_COMPLETE__CONDITION))) { LVL_ASCEND_COMPLETE__ACTION; goto S15;}
#ifndef LVL_SEARCH_ASCENDING__CONDITION
#define LVL_SEARCH_ASCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCENDING__ACTION
#define LVL_SEARCH_ASCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_ASCENDING__ACTION; TRACE_START__ACTION; goto S16;}
goto S_TRAPPED;
/*********/
S10:
fsm_current_state = "horizontal-search:lvl-search-descend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DESCEND_COMPLETE__CONDITION
#define LVL_DESCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_DESCEND_COMPLETE__ACTION
#define LVL_DESCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_DESCEND_COMPLETE__CONDITION))) { LVL_DESCEND_COMPLETE__ACTION; goto S17;}
#ifndef LVL_SEARCH_DESCENDING__CONDITION
#define LVL_SEARCH_DESCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCENDING__ACTION
#define LVL_SEARCH_DESCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_DESCENDING__ACTION; TRACE_START__ACTION; goto S18;}
goto S_TRAPPED;
/*********/
S9:
fsm_current_state = "horizontal-search:lvl-search-ascend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_ASCEND_COMPLETE__CONDITION
#define LVL_ASCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_ASCEND_COMPLETE__ACTION
#define LVL_ASCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_ASCEND_COMPLETE__CONDITION))) { LVL_ASCEND_COMPLETE__ACTION; goto S19;}
#ifndef LVL_SEARCH_ASCENDING__CONDITION
#define LVL_SEARCH_ASCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCENDING__ACTION
#define LVL_SEARCH_ASCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_ASCENDING__ACTION; TRACE_START__ACTION; goto S20;}
goto S_TRAPPED;
/*********/
S20:
fsm_current_state = "horizontal-search:lvl-search-ascending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_NORTH_TESTING_MARKS__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_NORTH_TESTING_MARKS__ACTION; goto S21;}
goto S_TRAPPED;
/*********/
S19:
fsm_current_state = "horizontal-search:lvl-ascend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_ABOVE__CONDITION
#define LVL_ABOVE__CONDITION (1)
#endif
#ifndef LVL_ABOVE__ACTION
#define LVL_ABOVE__ACTION { /* no-op */ }
#endif
#ifndef COLUMN_INCREASE__CONDITION
#define COLUMN_INCREASE__CONDITION (1)
#endif
#ifndef COLUMN_INCREASE__ACTION
#define COLUMN_INCREASE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_ABOVE__CONDITION) && (COLUMN_INCREASE__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_ABOVE__ACTION; COLUMN_INCREASE__ACTION; goto S22;}
goto S_TRAPPED;
/*********/
S18:
fsm_current_state = "horizontal-search:lvl-search-descending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_NORTH_TESTING_MARKS__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_NORTH_TESTING_MARKS__ACTION; goto S23;}
goto S_TRAPPED;
/*********/
S17:
fsm_current_state = "horizontal-search:lvl-descend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_BELOW__CONDITION
#define LVL_BELOW__CONDITION (1)
#endif
#ifndef LVL_BELOW__ACTION
#define LVL_BELOW__ACTION { /* no-op */ }
#endif
#ifndef COLUMN_INCREASE__CONDITION
#define COLUMN_INCREASE__CONDITION (1)
#endif
#ifndef COLUMN_INCREASE__ACTION
#define COLUMN_INCREASE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_BELOW__CONDITION) && (COLUMN_INCREASE__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_BELOW__ACTION; COLUMN_INCREASE__ACTION; goto S24;}
goto S_TRAPPED;
/*********/
S16:
fsm_current_state = "west-edge-search:lvl-search-ascending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_EAST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_EAST__ACTION; goto S25;}
goto S_TRAPPED;
/*********/
S15:
fsm_current_state = "west-edge-search:lvl-ascend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_INIT__CONDITION
#define HORIZONTAL_SEARCH_INIT__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_INIT__ACTION
#define HORIZONTAL_SEARCH_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_INIT__CONDITION))) { HORIZONTAL_SEARCH_INIT__ACTION; goto S2;}
goto S_TRAPPED;
/*********/
S14:
fsm_current_state = "west-edge-search:lvl-search-descending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_EAST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_EAST__ACTION; goto S26;}
goto S_TRAPPED;
/*********/
S13:
fsm_current_state = "west-edge-search:lvl-descend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_INIT__CONDITION
#define HORIZONTAL_SEARCH_INIT__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_INIT__ACTION
#define HORIZONTAL_SEARCH_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_INIT__CONDITION))) { HORIZONTAL_SEARCH_INIT__ACTION; goto S2;}
goto S_TRAPPED;
/*********/
S26:
fsm_current_state = "lvl-search-descending:west-edge-search:tracing:first-leg:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S27;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S28;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S29;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S30;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S31;}
goto S_TRAPPED;
/*********/
S25:
fsm_current_state = "lvl-search-ascending:west-edge-search:tracing:first-leg:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S32;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S33;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S34;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S35;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S36;}
goto S_TRAPPED;
/*********/
S24:
fsm_current_state = "horizontal-search:lvl-below:column-increase";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__CONDITION
#define HORIZONTAL_SEARCH_COMPLETE__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__ACTION
#define HORIZONTAL_SEARCH_COMPLETE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_COMPLETE__CONDITION))) { HORIZONTAL_SEARCH_COMPLETE__ACTION; goto S37;}
#ifndef CONTINUE_DOWNWARDS__CONDITION
#define CONTINUE_DOWNWARDS__CONDITION (1)
#endif
#ifndef CONTINUE_DOWNWARDS__ACTION
#define CONTINUE_DOWNWARDS__ACTION { /* no-op */ }
#endif
if (((CONTINUE_DOWNWARDS__CONDITION))) { CONTINUE_DOWNWARDS__ACTION; goto S38;}
#ifndef TURN_UPWARDS__CONDITION
#define TURN_UPWARDS__CONDITION (1)
#endif
#ifndef TURN_UPWARDS__ACTION
#define TURN_UPWARDS__ACTION { /* no-op */ }
#endif
if (((TURN_UPWARDS__CONDITION))) { TURN_UPWARDS__ACTION; goto S39;}
goto S_TRAPPED;
/*********/
S23:
fsm_current_state = "lvl-search-descending:horizontal-search:tracing:first-leg:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S40;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S41;}
goto S_TRAPPED;
/*********/
S22:
fsm_current_state = "horizontal-search:lvl-above:column-increase";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__CONDITION
#define HORIZONTAL_SEARCH_COMPLETE__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__ACTION
#define HORIZONTAL_SEARCH_COMPLETE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_COMPLETE__CONDITION))) { HORIZONTAL_SEARCH_COMPLETE__ACTION; goto S42;}
#ifndef CONTINUE_UPWARDS__CONDITION
#define CONTINUE_UPWARDS__CONDITION (1)
#endif
#ifndef CONTINUE_UPWARDS__ACTION
#define CONTINUE_UPWARDS__ACTION { /* no-op */ }
#endif
if (((CONTINUE_UPWARDS__CONDITION))) { CONTINUE_UPWARDS__ACTION; goto S43;}
#ifndef TURN_DOWNWARDS__CONDITION
#define TURN_DOWNWARDS__CONDITION (1)
#endif
#ifndef TURN_DOWNWARDS__ACTION
#define TURN_DOWNWARDS__ACTION { /* no-op */ }
#endif
if (((TURN_DOWNWARDS__CONDITION))) { TURN_DOWNWARDS__ACTION; goto S44;}
goto S_TRAPPED;
/*********/
S21:
fsm_current_state = "lvl-search-ascending:horizontal-search:tracing:first-leg:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S45;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S46;}
goto S_TRAPPED;
/*********/
S46:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S47;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S48;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S49;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S50;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S51;}
goto S_TRAPPED;
/*********/
S45:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S52;}
goto S_TRAPPED;
/*********/
S44:
fsm_current_state = "horizontal-search:turn-downwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S10;}
goto S_TRAPPED;
/*********/
S43:
fsm_current_state = "horizontal-search:continue-upwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S9;}
goto S_TRAPPED;
/*********/
S42:
fsm_current_state = "horizontal-search:lvl-above:horizontal-search-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
goto S_TRAPPED;
/*********/
S41:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S53;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S54;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S55;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S56;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S57;}
goto S_TRAPPED;
/*********/
S40:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S58;}
goto S_TRAPPED;
/*********/
S39:
fsm_current_state = "horizontal-search:turn-upwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S9;}
goto S_TRAPPED;
/*********/
S38:
fsm_current_state = "horizontal-search:continue-downwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S10;}
goto S_TRAPPED;
/*********/
S37:
fsm_current_state = "horizontal-search:lvl-below:horizontal-search-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
goto S_TRAPPED;
/*********/
S36:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S59;}
goto S_TRAPPED;
/*********/
S35:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S25;}
goto S_TRAPPED;
/*********/
S34:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S60;}
goto S_TRAPPED;
/*********/
S33:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S61;}
goto S_TRAPPED;
/*********/
S32:
fsm_current_state = "lvl-search-ascending:first-leg:west-edge-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S62;}
goto S_TRAPPED;
/*********/
S31:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S63;}
goto S_TRAPPED;
/*********/
S30:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S26;}
goto S_TRAPPED;
/*********/
S29:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S64;}
goto S_TRAPPED;
/*********/
S28:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S65;}
goto S_TRAPPED;
/*********/
S27:
fsm_current_state = "lvl-search-descending:first-leg:west-edge-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S66;}
goto S_TRAPPED;
/*********/
S66:
fsm_current_state = "lvl-search-descending:first-leg:west-edge-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S67;}
goto S_TRAPPED;
/*********/
S65:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef END_FIRST_LEG__CONDITION
#define END_FIRST_LEG__CONDITION (1)
#endif
#ifndef END_FIRST_LEG__ACTION
#define END_FIRST_LEG__ACTION { /* no-op */ }
#endif
if (((END_FIRST_LEG__CONDITION))) { END_FIRST_LEG__ACTION; goto S68;}
goto S_TRAPPED;
/*********/
S64:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S69;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S70;}
goto S_TRAPPED;
/*********/
S63:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S69;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S71;}
goto S_TRAPPED;
/*********/
S62:
fsm_current_state = "lvl-search-ascending:first-leg:west-edge-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S72;}
goto S_TRAPPED;
/*********/
S61:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef END_FIRST_LEG__CONDITION
#define END_FIRST_LEG__CONDITION (1)
#endif
#ifndef END_FIRST_LEG__ACTION
#define END_FIRST_LEG__ACTION { /* no-op */ }
#endif
if (((END_FIRST_LEG__CONDITION))) { END_FIRST_LEG__ACTION; goto S73;}
goto S_TRAPPED;
/*********/
S60:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S74;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S75;}
goto S_TRAPPED;
/*********/
S59:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S74;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S76;}
goto S_TRAPPED;
/*********/
S58:
fsm_current_state = "lvl-search-descending:horizontal-search:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S77;}
goto S_TRAPPED;
/*********/
S57:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S78;}
goto S_TRAPPED;
/*********/
S56:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S23;}
goto S_TRAPPED;
/*********/
S55:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S79;}
goto S_TRAPPED;
/*********/
S54:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S80;}
goto S_TRAPPED;
/*********/
S53:
fsm_current_state = "lvl-search-descending:first-leg:horizontal-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S81;}
goto S_TRAPPED;
/*********/
S52:
fsm_current_state = "lvl-search-ascending:horizontal-search:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S82;}
goto S_TRAPPED;
/*********/
S51:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S83;}
goto S_TRAPPED;
/*********/
S50:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S21;}
goto S_TRAPPED;
/*********/
S49:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S84;}
goto S_TRAPPED;
/*********/
S48:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S85;}
goto S_TRAPPED;
/*********/
S47:
fsm_current_state = "lvl-search-ascending:first-leg:horizontal-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S86;}
goto S_TRAPPED;
/*********/
S86:
fsm_current_state = "lvl-search-ascending:first-leg:horizontal-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S87;}
goto S_TRAPPED;
/*********/
S85:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef END_FIRST_LEG__CONDITION
#define END_FIRST_LEG__CONDITION (1)
#endif
#ifndef END_FIRST_LEG__ACTION
#define END_FIRST_LEG__ACTION { /* no-op */ }
#endif
if (((END_FIRST_LEG__CONDITION))) { END_FIRST_LEG__ACTION; goto S88;}
goto S_TRAPPED;
/*********/
S84:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S89;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S90;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S91;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S92;}
goto S_TRAPPED;
/*********/
S83:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S93;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S94;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S95;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S96;}
goto S_TRAPPED;
/*********/
S82:
fsm_current_state = "lvl-search-ascending:horizontal-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_INCREMENT__CONDITION
#define LVL_INCREMENT__CONDITION (1)
#endif
#ifndef LVL_INCREMENT__ACTION
#define LVL_INCREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_INCREMENT__CONDITION))) { LVL_INCREMENT__ACTION; goto S97;}
goto S_TRAPPED;
/*********/
S81:
fsm_current_state = "lvl-search-descending:first-leg:horizontal-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S98;}
goto S_TRAPPED;
/*********/
S80:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef END_FIRST_LEG__CONDITION
#define END_FIRST_LEG__CONDITION (1)
#endif
#ifndef END_FIRST_LEG__ACTION
#define END_FIRST_LEG__ACTION { /* no-op */ }
#endif
if (((END_FIRST_LEG__CONDITION))) { END_FIRST_LEG__ACTION; goto S99;}
goto S_TRAPPED;
/*********/
S79:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S100;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S101;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S102;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S103;}
goto S_TRAPPED;
/*********/
S78:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S104;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S105;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S106;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S107;}
goto S_TRAPPED;
/*********/
S77:
fsm_current_state = "lvl-search-descending:horizontal-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DECREMENT__CONDITION
#define LVL_DECREMENT__CONDITION (1)
#endif
#ifndef LVL_DECREMENT__ACTION
#define LVL_DECREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_DECREMENT__CONDITION))) { LVL_DECREMENT__ACTION; goto S108;}
goto S_TRAPPED;
/*********/
S76:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S109;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S110;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S111;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S112;}
goto S_TRAPPED;
/*********/
S75:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S113;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S114;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S115;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S116;}
goto S_TRAPPED;
/*********/
S74:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S117;}
goto S_TRAPPED;
/*********/
S73:
fsm_current_state = "lvl-search-ascending:west-edge-search:end-first-leg";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef SECOND_LEG__CONDITION
#define SECOND_LEG__CONDITION (1)
#endif
#ifndef SECOND_LEG__ACTION
#define SECOND_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (SECOND_LEG__CONDITION) && (FACING_WEST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; SECOND_LEG__ACTION; FACING_WEST__ACTION; goto S118;}
goto S_TRAPPED;
/*********/
S72:
fsm_current_state = "lvl-search-ascending:first-leg:west-edge-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_INCREMENT__CONDITION
#define LVL_INCREMENT__CONDITION (1)
#endif
#ifndef LVL_INCREMENT__ACTION
#define LVL_INCREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_INCREMENT__CONDITION))) { LVL_INCREMENT__ACTION; goto S119;}
goto S_TRAPPED;
/*********/
S71:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S120;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S121;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S122;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S123;}
goto S_TRAPPED;
/*********/
S70:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S124;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S125;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S126;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S127;}
goto S_TRAPPED;
/*********/
S69:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S128;}
goto S_TRAPPED;
/*********/
S68:
fsm_current_state = "lvl-search-descending:west-edge-search:end-first-leg";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef SECOND_LEG__CONDITION
#define SECOND_LEG__CONDITION (1)
#endif
#ifndef SECOND_LEG__ACTION
#define SECOND_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (SECOND_LEG__CONDITION) && (FACING_WEST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; SECOND_LEG__ACTION; FACING_WEST__ACTION; goto S129;}
goto S_TRAPPED;
/*********/
S67:
fsm_current_state = "lvl-search-descending:first-leg:west-edge-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DECREMENT__CONDITION
#define LVL_DECREMENT__CONDITION (1)
#endif
#ifndef LVL_DECREMENT__ACTION
#define LVL_DECREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_DECREMENT__CONDITION))) { LVL_DECREMENT__ACTION; goto S130;}
goto S_TRAPPED;
/*********/
S130:
fsm_current_state = "first-leg:west-edge-search:lvl-decrement";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S131;}
goto S_TRAPPED;
/*********/
S129:
fsm_current_state = "lvl-search-descending:west-edge-search:tracing:second-leg:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S132;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S133;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S134;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S135;}
goto S_TRAPPED;
/*********/
S128:
fsm_current_state = "lvl-search-descending:west-edge-search:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S67;}
goto S_TRAPPED;
/*********/
S127:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S136;}
goto S_TRAPPED;
/*********/
S126:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S64;}
goto S_TRAPPED;
/*********/
S125:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S26;}
goto S_TRAPPED;
/*********/
S124:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S65;}
goto S_TRAPPED;
/*********/
S123:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S26;}
goto S_TRAPPED;
/*********/
S122:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S63;}
goto S_TRAPPED;
/*********/
S121:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S136;}
goto S_TRAPPED;
/*********/
S120:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S65;}
goto S_TRAPPED;
/*********/
S119:
fsm_current_state = "first-leg:west-edge-search:lvl-increment";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S137;}
goto S_TRAPPED;
/*********/
S118:
fsm_current_state = "lvl-search-ascending:west-edge-search:tracing:second-leg:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S138;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S139;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S140;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S141;}
goto S_TRAPPED;
/*********/
S117:
fsm_current_state = "lvl-search-ascending:west-edge-search:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S72;}
goto S_TRAPPED;
/*********/
S116:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S142;}
goto S_TRAPPED;
/*********/
S115:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S60;}
goto S_TRAPPED;
/*********/
S114:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S25;}
goto S_TRAPPED;
/*********/
S113:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S61;}
goto S_TRAPPED;
/*********/
S112:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S25;}
goto S_TRAPPED;
/*********/
S111:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S59;}
goto S_TRAPPED;
/*********/
S110:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S142;}
goto S_TRAPPED;
/*********/
S109:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S61;}
goto S_TRAPPED;
/*********/
S108:
fsm_current_state = "horizontal-search:lvl-decrement";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S10;}
goto S_TRAPPED;
/*********/
S107:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S23;}
goto S_TRAPPED;
/*********/
S106:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S78;}
goto S_TRAPPED;
/*********/
S105:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S143;}
goto S_TRAPPED;
/*********/
S104:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S80;}
goto S_TRAPPED;
/*********/
S103:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S143;}
goto S_TRAPPED;
/*********/
S102:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S79;}
goto S_TRAPPED;
/*********/
S101:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S23;}
goto S_TRAPPED;
/*********/
S100:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S80;}
goto S_TRAPPED;
/*********/
S99:
fsm_current_state = "lvl-search-descending:horizontal-search:end-first-leg";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef SECOND_LEG__CONDITION
#define SECOND_LEG__CONDITION (1)
#endif
#ifndef SECOND_LEG__ACTION
#define SECOND_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (SECOND_LEG__CONDITION) && (FACING_SOUTH__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; SECOND_LEG__ACTION; FACING_SOUTH__ACTION; goto S144;}
goto S_TRAPPED;
/*********/
S98:
fsm_current_state = "lvl-search-descending:first-leg:horizontal-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DECREMENT__CONDITION
#define LVL_DECREMENT__CONDITION (1)
#endif
#ifndef LVL_DECREMENT__ACTION
#define LVL_DECREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_DECREMENT__CONDITION))) { LVL_DECREMENT__ACTION; goto S145;}
goto S_TRAPPED;
/*********/
S97:
fsm_current_state = "horizontal-search:lvl-increment";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S9;}
goto S_TRAPPED;
/*********/
S96:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S21;}
goto S_TRAPPED;
/*********/
S95:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S83;}
goto S_TRAPPED;
/*********/
S94:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S146;}
goto S_TRAPPED;
/*********/
S93:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S85;}
goto S_TRAPPED;
/*********/
S92:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S146;}
goto S_TRAPPED;
/*********/
S91:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S84;}
goto S_TRAPPED;
/*********/
S90:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S21;}
goto S_TRAPPED;
/*********/
S89:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S85;}
goto S_TRAPPED;
/*********/
S88:
fsm_current_state = "lvl-search-ascending:horizontal-search:end-first-leg";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef SECOND_LEG__CONDITION
#define SECOND_LEG__CONDITION (1)
#endif
#ifndef SECOND_LEG__ACTION
#define SECOND_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (SECOND_LEG__CONDITION) && (FACING_SOUTH__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; SECOND_LEG__ACTION; FACING_SOUTH__ACTION; goto S147;}
goto S_TRAPPED;
/*********/
S87:
fsm_current_state = "lvl-search-ascending:first-leg:horizontal-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_INCREMENT__CONDITION
#define LVL_INCREMENT__CONDITION (1)
#endif
#ifndef LVL_INCREMENT__ACTION
#define LVL_INCREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_INCREMENT__CONDITION))) { LVL_INCREMENT__ACTION; goto S148;}
goto S_TRAPPED;
/*********/
S148:
fsm_current_state = "first-leg:horizontal-search:lvl-increment";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S149;}
goto S_TRAPPED;
/*********/
S147:
fsm_current_state = "lvl-search-ascending:horizontal-search:tracing:second-leg:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S150;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S151;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S152;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S153;}
goto S_TRAPPED;
/*********/
S146:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S45;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S154;}
goto S_TRAPPED;
/*********/
S145:
fsm_current_state = "first-leg:horizontal-search:lvl-decrement";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S155;}
goto S_TRAPPED;
/*********/
S144:
fsm_current_state = "lvl-search-descending:horizontal-search:tracing:second-leg:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S156;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S157;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S158;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S159;}
goto S_TRAPPED;
/*********/
S143:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S40;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S160;}
goto S_TRAPPED;
/*********/
S142:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S161;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S162;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S163;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S164;}
goto S_TRAPPED;
/*********/
S141:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S165;}
goto S_TRAPPED;
/*********/
S140:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S118;}
goto S_TRAPPED;
/*********/
S139:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S166;}
goto S_TRAPPED;
/*********/
S138:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S167;}
goto S_TRAPPED;
/*********/
S137:
fsm_current_state = "first-leg:west-edge-search:lvl-search-ascend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_ASCEND_COMPLETE__CONDITION
#define LVL_ASCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_ASCEND_COMPLETE__ACTION
#define LVL_ASCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_ASCEND_COMPLETE__CONDITION))) { LVL_ASCEND_COMPLETE__ACTION; goto S168;}
#ifndef LVL_SEARCH_ASCENDING__CONDITION
#define LVL_SEARCH_ASCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCENDING__ACTION
#define LVL_SEARCH_ASCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_ASCENDING__ACTION; TRACE_START__ACTION; goto S169;}
goto S_TRAPPED;
/*********/
S136:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S170;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S171;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S172;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S173;}
goto S_TRAPPED;
/*********/
S135:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S174;}
goto S_TRAPPED;
/*********/
S134:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S129;}
goto S_TRAPPED;
/*********/
S133:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S175;}
goto S_TRAPPED;
/*********/
S132:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S176;}
goto S_TRAPPED;
/*********/
S131:
fsm_current_state = "first-leg:west-edge-search:lvl-search-descend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DESCEND_COMPLETE__CONDITION
#define LVL_DESCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_DESCEND_COMPLETE__ACTION
#define LVL_DESCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_DESCEND_COMPLETE__CONDITION))) { LVL_DESCEND_COMPLETE__ACTION; goto S177;}
#ifndef LVL_SEARCH_DESCENDING__CONDITION
#define LVL_SEARCH_DESCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCENDING__ACTION
#define LVL_SEARCH_DESCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_DESCENDING__ACTION; TRACE_START__ACTION; goto S178;}
goto S_TRAPPED;
/*********/
S178:
fsm_current_state = "first-leg:west-edge-search:lvl-search-descending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_EAST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_EAST__ACTION; goto S26;}
goto S_TRAPPED;
/*********/
S177:
fsm_current_state = "first-leg:west-edge-search:lvl-descend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_INIT__CONDITION
#define HORIZONTAL_SEARCH_INIT__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_INIT__ACTION
#define HORIZONTAL_SEARCH_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_INIT__CONDITION))) { HORIZONTAL_SEARCH_INIT__ACTION; goto S179;}
goto S_TRAPPED;
/*********/
S176:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_OPEN__CONDITION
#define TRACE_DRAW_OPEN__CONDITION (1)
#endif
#ifndef TRACE_DRAW_OPEN__ACTION
#define TRACE_DRAW_OPEN__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_OPEN__CONDITION))) { TRACE_DRAW_OPEN__ACTION; goto S180;}
goto S_TRAPPED;
/*********/
S175:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S181;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S182;}
goto S_TRAPPED;
/*********/
S174:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S181;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S183;}
goto S_TRAPPED;
/*********/
S173:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S64;}
goto S_TRAPPED;
/*********/
S172:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S136;}
goto S_TRAPPED;
/*********/
S171:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S63;}
goto S_TRAPPED;
/*********/
S170:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S65;}
goto S_TRAPPED;
/*********/
S169:
fsm_current_state = "first-leg:west-edge-search:lvl-search-ascending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_EAST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_EAST__ACTION; goto S25;}
goto S_TRAPPED;
/*********/
S168:
fsm_current_state = "first-leg:west-edge-search:lvl-ascend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_INIT__CONDITION
#define HORIZONTAL_SEARCH_INIT__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_INIT__ACTION
#define HORIZONTAL_SEARCH_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_INIT__CONDITION))) { HORIZONTAL_SEARCH_INIT__ACTION; goto S179;}
goto S_TRAPPED;
/*********/
S167:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_OPEN__CONDITION
#define TRACE_DRAW_OPEN__CONDITION (1)
#endif
#ifndef TRACE_DRAW_OPEN__ACTION
#define TRACE_DRAW_OPEN__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_OPEN__CONDITION))) { TRACE_DRAW_OPEN__ACTION; goto S184;}
goto S_TRAPPED;
/*********/
S166:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S185;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S186;}
goto S_TRAPPED;
/*********/
S165:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S185;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S187;}
goto S_TRAPPED;
/*********/
S164:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S60;}
goto S_TRAPPED;
/*********/
S163:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S142;}
goto S_TRAPPED;
/*********/
S162:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S59;}
goto S_TRAPPED;
/*********/
S161:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S61;}
goto S_TRAPPED;
/*********/
S160:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S188;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S189;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S190;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S191;}
goto S_TRAPPED;
/*********/
S159:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S192;}
goto S_TRAPPED;
/*********/
S158:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S193;}
goto S_TRAPPED;
/*********/
S157:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S194;}
goto S_TRAPPED;
/*********/
S156:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S195;}
goto S_TRAPPED;
/*********/
S155:
fsm_current_state = "first-leg:horizontal-search:lvl-search-descend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DESCEND_COMPLETE__CONDITION
#define LVL_DESCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_DESCEND_COMPLETE__ACTION
#define LVL_DESCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_DESCEND_COMPLETE__CONDITION))) { LVL_DESCEND_COMPLETE__ACTION; goto S196;}
#ifndef LVL_SEARCH_DESCENDING__CONDITION
#define LVL_SEARCH_DESCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCENDING__ACTION
#define LVL_SEARCH_DESCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_DESCENDING__ACTION; TRACE_START__ACTION; goto S197;}
goto S_TRAPPED;
/*********/
S154:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S198;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S199;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S200;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S201;}
goto S_TRAPPED;
/*********/
S153:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S202;}
goto S_TRAPPED;
/*********/
S152:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S203;}
goto S_TRAPPED;
/*********/
S151:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S204;}
goto S_TRAPPED;
/*********/
S150:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S205;}
goto S_TRAPPED;
/*********/
S149:
fsm_current_state = "first-leg:horizontal-search:lvl-search-ascend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_ASCEND_COMPLETE__CONDITION
#define LVL_ASCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_ASCEND_COMPLETE__ACTION
#define LVL_ASCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_ASCEND_COMPLETE__CONDITION))) { LVL_ASCEND_COMPLETE__ACTION; goto S206;}
#ifndef LVL_SEARCH_ASCENDING__CONDITION
#define LVL_SEARCH_ASCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCENDING__ACTION
#define LVL_SEARCH_ASCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_ASCENDING__ACTION; TRACE_START__ACTION; goto S207;}
goto S_TRAPPED;
/*********/
S207:
fsm_current_state = "first-leg:horizontal-search:lvl-search-ascending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_NORTH_TESTING_MARKS__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_NORTH_TESTING_MARKS__ACTION; goto S21;}
goto S_TRAPPED;
/*********/
S206:
fsm_current_state = "first-leg:horizontal-search:lvl-ascend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_ABOVE__CONDITION
#define LVL_ABOVE__CONDITION (1)
#endif
#ifndef LVL_ABOVE__ACTION
#define LVL_ABOVE__ACTION { /* no-op */ }
#endif
#ifndef COLUMN_INCREASE__CONDITION
#define COLUMN_INCREASE__CONDITION (1)
#endif
#ifndef COLUMN_INCREASE__ACTION
#define COLUMN_INCREASE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_ABOVE__CONDITION) && (COLUMN_INCREASE__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_ABOVE__ACTION; COLUMN_INCREASE__ACTION; goto S208;}
goto S_TRAPPED;
/*********/
S205:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_OPEN__CONDITION
#define TRACE_DRAW_OPEN__CONDITION (1)
#endif
#ifndef TRACE_DRAW_OPEN__ACTION
#define TRACE_DRAW_OPEN__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_OPEN__CONDITION))) { TRACE_DRAW_OPEN__ACTION; goto S209;}
goto S_TRAPPED;
/*********/
S204:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S210;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S211;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S212;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S213;}
goto S_TRAPPED;
/*********/
S203:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S214;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S147;}
goto S_TRAPPED;
/*********/
S202:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S215;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S216;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S217;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S218;}
goto S_TRAPPED;
/*********/
S201:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S84;}
goto S_TRAPPED;
/*********/
S200:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S146;}
goto S_TRAPPED;
/*********/
S199:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S83;}
goto S_TRAPPED;
/*********/
S198:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S85;}
goto S_TRAPPED;
/*********/
S197:
fsm_current_state = "first-leg:horizontal-search:lvl-search-descending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_NORTH_TESTING_MARKS__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_NORTH_TESTING_MARKS__ACTION; goto S23;}
goto S_TRAPPED;
/*********/
S196:
fsm_current_state = "first-leg:horizontal-search:lvl-descend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_BELOW__CONDITION
#define LVL_BELOW__CONDITION (1)
#endif
#ifndef LVL_BELOW__ACTION
#define LVL_BELOW__ACTION { /* no-op */ }
#endif
#ifndef COLUMN_INCREASE__CONDITION
#define COLUMN_INCREASE__CONDITION (1)
#endif
#ifndef COLUMN_INCREASE__ACTION
#define COLUMN_INCREASE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_BELOW__CONDITION) && (COLUMN_INCREASE__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_BELOW__ACTION; COLUMN_INCREASE__ACTION; goto S219;}
goto S_TRAPPED;
/*********/
S195:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_OPEN__CONDITION
#define TRACE_DRAW_OPEN__CONDITION (1)
#endif
#ifndef TRACE_DRAW_OPEN__ACTION
#define TRACE_DRAW_OPEN__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_OPEN__CONDITION))) { TRACE_DRAW_OPEN__ACTION; goto S220;}
goto S_TRAPPED;
/*********/
S194:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S221;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S222;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S223;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S224;}
goto S_TRAPPED;
/*********/
S193:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S225;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S144;}
goto S_TRAPPED;
/*********/
S192:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S226;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S227;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S228;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S229;}
goto S_TRAPPED;
/*********/
S191:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S79;}
goto S_TRAPPED;
/*********/
S190:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S143;}
goto S_TRAPPED;
/*********/
S189:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S78;}
goto S_TRAPPED;
/*********/
S188:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S80;}
goto S_TRAPPED;
/*********/
S187:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S230;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S231;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S232;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S233;}
goto S_TRAPPED;
/*********/
S186:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S234;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S235;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S236;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S237;}
goto S_TRAPPED;
/*********/
S185:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S117;}
goto S_TRAPPED;
/*********/
S184:
fsm_current_state = "lvl-search-ascending:west-edge-search:trace-draw-open";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S72;}
goto S_TRAPPED;
/*********/
S183:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S238;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S239;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S240;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S241;}
goto S_TRAPPED;
/*********/
S182:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S242;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S243;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S244;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S245;}
goto S_TRAPPED;
/*********/
S181:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S128;}
goto S_TRAPPED;
/*********/
S180:
fsm_current_state = "lvl-search-descending:west-edge-search:trace-draw-open";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S67;}
goto S_TRAPPED;
/*********/
S179:
fsm_current_state = "first-leg:horizontal-search-init";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_SEEK_INIT__CONDITION
#define LVL_SEEK_INIT__CONDITION (1)
#endif
#ifndef LVL_SEEK_INIT__ACTION
#define LVL_SEEK_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_SEEK_INIT__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_SEEK_INIT__ACTION; goto S246;}
goto S_TRAPPED;
/*********/
S246:
fsm_current_state = "first-leg:horizontal-search:lvl-seek-init";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEEK_FROM_TOP__CONDITION
#define LVL_SEEK_FROM_TOP__CONDITION (1)
#endif
#ifndef LVL_SEEK_FROM_TOP__ACTION
#define LVL_SEEK_FROM_TOP__ACTION { /* no-op */ }
#endif
if (((LVL_SEEK_FROM_TOP__CONDITION))) { LVL_SEEK_FROM_TOP__ACTION; goto S247;}
#ifndef LVL_SEEK_FROM_BOTTOM__CONDITION
#define LVL_SEEK_FROM_BOTTOM__CONDITION (1)
#endif
#ifndef LVL_SEEK_FROM_BOTTOM__ACTION
#define LVL_SEEK_FROM_BOTTOM__ACTION { /* no-op */ }
#endif
if (((LVL_SEEK_FROM_BOTTOM__CONDITION))) { LVL_SEEK_FROM_BOTTOM__ACTION; goto S248;}
goto S_TRAPPED;
/*********/
S245:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S249;}
goto S_TRAPPED;
/*********/
S244:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S175;}
goto S_TRAPPED;
/*********/
S243:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S129;}
goto S_TRAPPED;
/*********/
S242:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S176;}
goto S_TRAPPED;
/*********/
S241:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S129;}
goto S_TRAPPED;
/*********/
S240:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S174;}
goto S_TRAPPED;
/*********/
S239:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S249;}
goto S_TRAPPED;
/*********/
S238:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S176;}
goto S_TRAPPED;
/*********/
S237:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S250;}
goto S_TRAPPED;
/*********/
S236:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S166;}
goto S_TRAPPED;
/*********/
S235:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S118;}
goto S_TRAPPED;
/*********/
S234:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S167;}
goto S_TRAPPED;
/*********/
S233:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S118;}
goto S_TRAPPED;
/*********/
S232:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S165;}
goto S_TRAPPED;
/*********/
S231:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S250;}
goto S_TRAPPED;
/*********/
S230:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S167;}
goto S_TRAPPED;
/*********/
S229:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S193;}
goto S_TRAPPED;
/*********/
S228:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S192;}
goto S_TRAPPED;
/*********/
S227:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S251;}
goto S_TRAPPED;
/*********/
S226:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S195;}
goto S_TRAPPED;
/*********/
S225:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S58;}
goto S_TRAPPED;
/*********/
S224:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S251;}
goto S_TRAPPED;
/*********/
S223:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S194;}
goto S_TRAPPED;
/*********/
S222:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S193;}
goto S_TRAPPED;
/*********/
S221:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S195;}
goto S_TRAPPED;
/*********/
S220:
fsm_current_state = "lvl-search-descending:horizontal-search:trace-draw-open";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S98;}
goto S_TRAPPED;
/*********/
S219:
fsm_current_state = "first-leg:horizontal-search:lvl-below:column-increase";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__CONDITION
#define HORIZONTAL_SEARCH_COMPLETE__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__ACTION
#define HORIZONTAL_SEARCH_COMPLETE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_COMPLETE__CONDITION))) { HORIZONTAL_SEARCH_COMPLETE__ACTION; goto S252;}
#ifndef CONTINUE_DOWNWARDS__CONDITION
#define CONTINUE_DOWNWARDS__CONDITION (1)
#endif
#ifndef CONTINUE_DOWNWARDS__ACTION
#define CONTINUE_DOWNWARDS__ACTION { /* no-op */ }
#endif
if (((CONTINUE_DOWNWARDS__CONDITION))) { CONTINUE_DOWNWARDS__ACTION; goto S253;}
#ifndef TURN_UPWARDS__CONDITION
#define TURN_UPWARDS__CONDITION (1)
#endif
#ifndef TURN_UPWARDS__ACTION
#define TURN_UPWARDS__ACTION { /* no-op */ }
#endif
if (((TURN_UPWARDS__CONDITION))) { TURN_UPWARDS__ACTION; goto S254;}
goto S_TRAPPED;
/*********/
S218:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S203;}
goto S_TRAPPED;
/*********/
S217:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S202;}
goto S_TRAPPED;
/*********/
S216:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S255;}
goto S_TRAPPED;
/*********/
S215:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S205;}
goto S_TRAPPED;
/*********/
S214:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S52;}
goto S_TRAPPED;
/*********/
S213:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S255;}
goto S_TRAPPED;
/*********/
S212:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S204;}
goto S_TRAPPED;
/*********/
S211:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S203;}
goto S_TRAPPED;
/*********/
S210:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S205;}
goto S_TRAPPED;
/*********/
S209:
fsm_current_state = "lvl-search-ascending:horizontal-search:trace-draw-open";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S87;}
goto S_TRAPPED;
/*********/
S208:
fsm_current_state = "first-leg:horizontal-search:lvl-above:column-increase";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__CONDITION
#define HORIZONTAL_SEARCH_COMPLETE__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__ACTION
#define HORIZONTAL_SEARCH_COMPLETE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_COMPLETE__CONDITION))) { HORIZONTAL_SEARCH_COMPLETE__ACTION; goto S256;}
#ifndef CONTINUE_UPWARDS__CONDITION
#define CONTINUE_UPWARDS__CONDITION (1)
#endif
#ifndef CONTINUE_UPWARDS__ACTION
#define CONTINUE_UPWARDS__ACTION { /* no-op */ }
#endif
if (((CONTINUE_UPWARDS__CONDITION))) { CONTINUE_UPWARDS__ACTION; goto S257;}
#ifndef TURN_DOWNWARDS__CONDITION
#define TURN_DOWNWARDS__CONDITION (1)
#endif
#ifndef TURN_DOWNWARDS__ACTION
#define TURN_DOWNWARDS__ACTION { /* no-op */ }
#endif
if (((TURN_DOWNWARDS__CONDITION))) { TURN_DOWNWARDS__ACTION; goto S258;}
goto S_TRAPPED;
/*********/
S258:
fsm_current_state = "first-leg:horizontal-search:turn-downwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S155;}
goto S_TRAPPED;
/*********/
S257:
fsm_current_state = "first-leg:horizontal-search:continue-upwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S149;}
goto S_TRAPPED;
/*********/
S256:
fsm_current_state = "first-leg:horizontal-search:lvl-above:horizontal-search-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
goto S_TRAPPED;
/*********/
S255:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S214;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S259;}
goto S_TRAPPED;
/*********/
S254:
fsm_current_state = "first-leg:horizontal-search:turn-upwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S149;}
goto S_TRAPPED;
/*********/
S253:
fsm_current_state = "first-leg:horizontal-search:continue-downwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S155;}
goto S_TRAPPED;
/*********/
S252:
fsm_current_state = "first-leg:horizontal-search:lvl-below:horizontal-search-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
goto S_TRAPPED;
/*********/
S251:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S225;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S260;}
goto S_TRAPPED;
/*********/
S250:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S261;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S262;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S263;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S264;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S265;}
goto S_TRAPPED;
/*********/
S249:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S266;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S267;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S268;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S269;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S270;}
goto S_TRAPPED;
/*********/
S248:
fsm_current_state = "first-leg:horizontal-search:lvl-seek-from-bottom";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S149;}
goto S_TRAPPED;
/*********/
S247:
fsm_current_state = "first-leg:horizontal-search:lvl-seek-from-top";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S155;}
goto S_TRAPPED;
/*********/
S270:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S175;}
goto S_TRAPPED;
/*********/
S269:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S249;}
goto S_TRAPPED;
/*********/
S268:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S174;}
goto S_TRAPPED;
/*********/
S267:
fsm_current_state = "lvl-search-descending:west-edge-search:second-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S176;}
goto S_TRAPPED;
/*********/
S266:
fsm_current_state = "lvl-search-descending:second-leg:west-edge-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S271;}
goto S_TRAPPED;
/*********/
S265:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S166;}
goto S_TRAPPED;
/*********/
S264:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S250;}
goto S_TRAPPED;
/*********/
S263:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S165;}
goto S_TRAPPED;
/*********/
S262:
fsm_current_state = "lvl-search-ascending:west-edge-search:second-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S167;}
goto S_TRAPPED;
/*********/
S261:
fsm_current_state = "lvl-search-ascending:second-leg:west-edge-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S272;}
goto S_TRAPPED;
/*********/
S260:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S273;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S274;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S275;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S276;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S277;}
goto S_TRAPPED;
/*********/
S259:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S278;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S279;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S280;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S281;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S282;}
goto S_TRAPPED;
/*********/
S282:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S204;}
goto S_TRAPPED;
/*********/
S281:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S255;}
goto S_TRAPPED;
/*********/
S280:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S202;}
goto S_TRAPPED;
/*********/
S279:
fsm_current_state = "lvl-search-ascending:horizontal-search:second-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S205;}
goto S_TRAPPED;
/*********/
S278:
fsm_current_state = "lvl-search-ascending:second-leg:horizontal-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S283;}
goto S_TRAPPED;
/*********/
S277:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S194;}
goto S_TRAPPED;
/*********/
S276:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S251;}
goto S_TRAPPED;
/*********/
S275:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S192;}
goto S_TRAPPED;
/*********/
S274:
fsm_current_state = "lvl-search-descending:horizontal-search:second-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S195;}
goto S_TRAPPED;
/*********/
S273:
fsm_current_state = "lvl-search-descending:second-leg:horizontal-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S284;}
goto S_TRAPPED;
/*********/
S272:
fsm_current_state = "lvl-search-ascending:second-leg:west-edge-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S285;}
goto S_TRAPPED;
/*********/
S271:
fsm_current_state = "lvl-search-descending:second-leg:west-edge-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S286;}
goto S_TRAPPED;
/*********/
S286:
fsm_current_state = "lvl-search-descending:second-leg:west-edge-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DECREMENT__CONDITION
#define LVL_DECREMENT__CONDITION (1)
#endif
#ifndef LVL_DECREMENT__ACTION
#define LVL_DECREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_DECREMENT__CONDITION))) { LVL_DECREMENT__ACTION; goto S287;}
goto S_TRAPPED;
/*********/
S285:
fsm_current_state = "lvl-search-ascending:second-leg:west-edge-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_INCREMENT__CONDITION
#define LVL_INCREMENT__CONDITION (1)
#endif
#ifndef LVL_INCREMENT__ACTION
#define LVL_INCREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_INCREMENT__CONDITION))) { LVL_INCREMENT__ACTION; goto S288;}
goto S_TRAPPED;
/*********/
S284:
fsm_current_state = "lvl-search-descending:second-leg:horizontal-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S289;}
goto S_TRAPPED;
/*********/
S283:
fsm_current_state = "lvl-search-ascending:second-leg:horizontal-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S290;}
goto S_TRAPPED;
/*********/
S290:
fsm_current_state = "lvl-search-ascending:second-leg:horizontal-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_INCREMENT__CONDITION
#define LVL_INCREMENT__CONDITION (1)
#endif
#ifndef LVL_INCREMENT__ACTION
#define LVL_INCREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_INCREMENT__CONDITION))) { LVL_INCREMENT__ACTION; goto S291;}
goto S_TRAPPED;
/*********/
S289:
fsm_current_state = "lvl-search-descending:second-leg:horizontal-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DECREMENT__CONDITION
#define LVL_DECREMENT__CONDITION (1)
#endif
#ifndef LVL_DECREMENT__ACTION
#define LVL_DECREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_DECREMENT__CONDITION))) { LVL_DECREMENT__ACTION; goto S292;}
goto S_TRAPPED;
/*********/
S288:
fsm_current_state = "second-leg:west-edge-search:lvl-increment";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S293;}
goto S_TRAPPED;
/*********/
S287:
fsm_current_state = "second-leg:west-edge-search:lvl-decrement";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S294;}
goto S_TRAPPED;
/*********/
S294:
fsm_current_state = "second-leg:west-edge-search:lvl-search-descend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DESCEND_COMPLETE__CONDITION
#define LVL_DESCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_DESCEND_COMPLETE__ACTION
#define LVL_DESCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_DESCEND_COMPLETE__CONDITION))) { LVL_DESCEND_COMPLETE__ACTION; goto S295;}
#ifndef LVL_SEARCH_DESCENDING__CONDITION
#define LVL_SEARCH_DESCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCENDING__ACTION
#define LVL_SEARCH_DESCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_DESCENDING__ACTION; TRACE_START__ACTION; goto S296;}
goto S_TRAPPED;
/*********/
S293:
fsm_current_state = "second-leg:west-edge-search:lvl-search-ascend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_ASCEND_COMPLETE__CONDITION
#define LVL_ASCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_ASCEND_COMPLETE__ACTION
#define LVL_ASCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_ASCEND_COMPLETE__CONDITION))) { LVL_ASCEND_COMPLETE__ACTION; goto S297;}
#ifndef LVL_SEARCH_ASCENDING__CONDITION
#define LVL_SEARCH_ASCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCENDING__ACTION
#define LVL_SEARCH_ASCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_ASCENDING__ACTION; TRACE_START__ACTION; goto S298;}
goto S_TRAPPED;
/*********/
S292:
fsm_current_state = "second-leg:horizontal-search:lvl-decrement";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S299;}
goto S_TRAPPED;
/*********/
S291:
fsm_current_state = "second-leg:horizontal-search:lvl-increment";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S300;}
goto S_TRAPPED;
/*********/
S300:
fsm_current_state = "second-leg:horizontal-search:lvl-search-ascend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_ASCEND_COMPLETE__CONDITION
#define LVL_ASCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_ASCEND_COMPLETE__ACTION
#define LVL_ASCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_ASCEND_COMPLETE__CONDITION))) { LVL_ASCEND_COMPLETE__ACTION; goto S301;}
#ifndef LVL_SEARCH_ASCENDING__CONDITION
#define LVL_SEARCH_ASCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCENDING__ACTION
#define LVL_SEARCH_ASCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_ASCENDING__ACTION; TRACE_START__ACTION; goto S302;}
goto S_TRAPPED;
/*********/
S299:
fsm_current_state = "second-leg:horizontal-search:lvl-search-descend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DESCEND_COMPLETE__CONDITION
#define LVL_DESCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_DESCEND_COMPLETE__ACTION
#define LVL_DESCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_DESCEND_COMPLETE__CONDITION))) { LVL_DESCEND_COMPLETE__ACTION; goto S303;}
#ifndef LVL_SEARCH_DESCENDING__CONDITION
#define LVL_SEARCH_DESCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCENDING__ACTION
#define LVL_SEARCH_DESCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_DESCENDING__ACTION; TRACE_START__ACTION; goto S304;}
goto S_TRAPPED;
/*********/
S298:
fsm_current_state = "second-leg:west-edge-search:lvl-search-ascending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_EAST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_EAST__ACTION; goto S305;}
goto S_TRAPPED;
/*********/
S297:
fsm_current_state = "second-leg:west-edge-search:lvl-ascend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_INIT__CONDITION
#define HORIZONTAL_SEARCH_INIT__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_INIT__ACTION
#define HORIZONTAL_SEARCH_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_INIT__CONDITION))) { HORIZONTAL_SEARCH_INIT__ACTION; goto S306;}
goto S_TRAPPED;
/*********/
S296:
fsm_current_state = "second-leg:west-edge-search:lvl-search-descending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_EAST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_EAST__ACTION; goto S307;}
goto S_TRAPPED;
/*********/
S295:
fsm_current_state = "second-leg:west-edge-search:lvl-descend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_INIT__CONDITION
#define HORIZONTAL_SEARCH_INIT__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_INIT__ACTION
#define HORIZONTAL_SEARCH_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_INIT__CONDITION))) { HORIZONTAL_SEARCH_INIT__ACTION; goto S306;}
goto S_TRAPPED;
/*********/
S307:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:tracing:first-leg:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S308;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S309;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S310;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S311;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S312;}
goto S_TRAPPED;
/*********/
S306:
fsm_current_state = "second-leg:horizontal-search-init";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_SEEK_INIT__CONDITION
#define LVL_SEEK_INIT__CONDITION (1)
#endif
#ifndef LVL_SEEK_INIT__ACTION
#define LVL_SEEK_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_SEEK_INIT__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_SEEK_INIT__ACTION; goto S313;}
goto S_TRAPPED;
/*********/
S305:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:tracing:first-leg:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S314;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S315;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S316;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S317;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S318;}
goto S_TRAPPED;
/*********/
S304:
fsm_current_state = "second-leg:horizontal-search:lvl-search-descending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_NORTH_TESTING_MARKS__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_NORTH_TESTING_MARKS__ACTION; goto S319;}
goto S_TRAPPED;
/*********/
S303:
fsm_current_state = "second-leg:horizontal-search:lvl-descend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_BELOW__CONDITION
#define LVL_BELOW__CONDITION (1)
#endif
#ifndef LVL_BELOW__ACTION
#define LVL_BELOW__ACTION { /* no-op */ }
#endif
#ifndef COLUMN_INCREASE__CONDITION
#define COLUMN_INCREASE__CONDITION (1)
#endif
#ifndef COLUMN_INCREASE__ACTION
#define COLUMN_INCREASE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_BELOW__CONDITION) && (COLUMN_INCREASE__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_BELOW__ACTION; COLUMN_INCREASE__ACTION; goto S320;}
goto S_TRAPPED;
/*********/
S302:
fsm_current_state = "second-leg:horizontal-search:lvl-search-ascending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_NORTH_TESTING_MARKS__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_NORTH_TESTING_MARKS__ACTION; goto S321;}
goto S_TRAPPED;
/*********/
S301:
fsm_current_state = "second-leg:horizontal-search:lvl-ascend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_ABOVE__CONDITION
#define LVL_ABOVE__CONDITION (1)
#endif
#ifndef LVL_ABOVE__ACTION
#define LVL_ABOVE__ACTION { /* no-op */ }
#endif
#ifndef COLUMN_INCREASE__CONDITION
#define COLUMN_INCREASE__CONDITION (1)
#endif
#ifndef COLUMN_INCREASE__ACTION
#define COLUMN_INCREASE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_ABOVE__CONDITION) && (COLUMN_INCREASE__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_ABOVE__ACTION; COLUMN_INCREASE__ACTION; goto S322;}
goto S_TRAPPED;
/*********/
S322:
fsm_current_state = "second-leg:horizontal-search:lvl-above:column-increase";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__CONDITION
#define HORIZONTAL_SEARCH_COMPLETE__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__ACTION
#define HORIZONTAL_SEARCH_COMPLETE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_COMPLETE__CONDITION))) { HORIZONTAL_SEARCH_COMPLETE__ACTION; goto S323;}
#ifndef CONTINUE_UPWARDS__CONDITION
#define CONTINUE_UPWARDS__CONDITION (1)
#endif
#ifndef CONTINUE_UPWARDS__ACTION
#define CONTINUE_UPWARDS__ACTION { /* no-op */ }
#endif
if (((CONTINUE_UPWARDS__CONDITION))) { CONTINUE_UPWARDS__ACTION; goto S324;}
#ifndef TURN_DOWNWARDS__CONDITION
#define TURN_DOWNWARDS__CONDITION (1)
#endif
#ifndef TURN_DOWNWARDS__ACTION
#define TURN_DOWNWARDS__ACTION { /* no-op */ }
#endif
if (((TURN_DOWNWARDS__CONDITION))) { TURN_DOWNWARDS__ACTION; goto S325;}
goto S_TRAPPED;
/*********/
S321:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:tracing:first-leg:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S326;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S327;}
goto S_TRAPPED;
/*********/
S320:
fsm_current_state = "second-leg:horizontal-search:lvl-below:column-increase";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__CONDITION
#define HORIZONTAL_SEARCH_COMPLETE__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__ACTION
#define HORIZONTAL_SEARCH_COMPLETE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_COMPLETE__CONDITION))) { HORIZONTAL_SEARCH_COMPLETE__ACTION; goto S328;}
#ifndef CONTINUE_DOWNWARDS__CONDITION
#define CONTINUE_DOWNWARDS__CONDITION (1)
#endif
#ifndef CONTINUE_DOWNWARDS__ACTION
#define CONTINUE_DOWNWARDS__ACTION { /* no-op */ }
#endif
if (((CONTINUE_DOWNWARDS__CONDITION))) { CONTINUE_DOWNWARDS__ACTION; goto S329;}
#ifndef TURN_UPWARDS__CONDITION
#define TURN_UPWARDS__CONDITION (1)
#endif
#ifndef TURN_UPWARDS__ACTION
#define TURN_UPWARDS__ACTION { /* no-op */ }
#endif
if (((TURN_UPWARDS__CONDITION))) { TURN_UPWARDS__ACTION; goto S330;}
goto S_TRAPPED;
/*********/
S319:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:tracing:first-leg:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S331;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S332;}
goto S_TRAPPED;
/*********/
S318:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S333;}
goto S_TRAPPED;
/*********/
S317:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S305;}
goto S_TRAPPED;
/*********/
S316:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S334;}
goto S_TRAPPED;
/*********/
S315:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S335;}
goto S_TRAPPED;
/*********/
S314:
fsm_current_state = "second-leg:lvl-search-ascending:first-leg:west-edge-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S336;}
goto S_TRAPPED;
/*********/
S313:
fsm_current_state = "second-leg:horizontal-search:lvl-seek-init";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEEK_FROM_TOP__CONDITION
#define LVL_SEEK_FROM_TOP__CONDITION (1)
#endif
#ifndef LVL_SEEK_FROM_TOP__ACTION
#define LVL_SEEK_FROM_TOP__ACTION { /* no-op */ }
#endif
if (((LVL_SEEK_FROM_TOP__CONDITION))) { LVL_SEEK_FROM_TOP__ACTION; goto S337;}
#ifndef LVL_SEEK_FROM_BOTTOM__CONDITION
#define LVL_SEEK_FROM_BOTTOM__CONDITION (1)
#endif
#ifndef LVL_SEEK_FROM_BOTTOM__ACTION
#define LVL_SEEK_FROM_BOTTOM__ACTION { /* no-op */ }
#endif
if (((LVL_SEEK_FROM_BOTTOM__CONDITION))) { LVL_SEEK_FROM_BOTTOM__ACTION; goto S338;}
goto S_TRAPPED;
/*********/
S312:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S339;}
goto S_TRAPPED;
/*********/
S311:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S307;}
goto S_TRAPPED;
/*********/
S310:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S340;}
goto S_TRAPPED;
/*********/
S309:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S341;}
goto S_TRAPPED;
/*********/
S308:
fsm_current_state = "second-leg:lvl-search-descending:first-leg:west-edge-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S342;}
goto S_TRAPPED;
/*********/
S342:
fsm_current_state = "second-leg:lvl-search-descending:first-leg:west-edge-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S343;}
goto S_TRAPPED;
/*********/
S341:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef END_FIRST_LEG__CONDITION
#define END_FIRST_LEG__CONDITION (1)
#endif
#ifndef END_FIRST_LEG__ACTION
#define END_FIRST_LEG__ACTION { /* no-op */ }
#endif
if (((END_FIRST_LEG__CONDITION))) { END_FIRST_LEG__ACTION; goto S344;}
#ifndef TRACE_DRAW_OPEN__CONDITION
#define TRACE_DRAW_OPEN__CONDITION (1)
#endif
#ifndef TRACE_DRAW_OPEN__ACTION
#define TRACE_DRAW_OPEN__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_OPEN__CONDITION))) { TRACE_DRAW_OPEN__ACTION; goto S345;}
goto S_TRAPPED;
/*********/
S340:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S346;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S347;}
goto S_TRAPPED;
/*********/
S339:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S346;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S348;}
goto S_TRAPPED;
/*********/
S338:
fsm_current_state = "second-leg:horizontal-search:lvl-seek-from-bottom";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S300;}
goto S_TRAPPED;
/*********/
S337:
fsm_current_state = "second-leg:horizontal-search:lvl-seek-from-top";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S299;}
goto S_TRAPPED;
/*********/
S336:
fsm_current_state = "second-leg:lvl-search-ascending:first-leg:west-edge-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S349;}
goto S_TRAPPED;
/*********/
S335:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef END_FIRST_LEG__CONDITION
#define END_FIRST_LEG__CONDITION (1)
#endif
#ifndef END_FIRST_LEG__ACTION
#define END_FIRST_LEG__ACTION { /* no-op */ }
#endif
if (((END_FIRST_LEG__CONDITION))) { END_FIRST_LEG__ACTION; goto S350;}
#ifndef TRACE_DRAW_OPEN__CONDITION
#define TRACE_DRAW_OPEN__CONDITION (1)
#endif
#ifndef TRACE_DRAW_OPEN__ACTION
#define TRACE_DRAW_OPEN__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_OPEN__CONDITION))) { TRACE_DRAW_OPEN__ACTION; goto S351;}
goto S_TRAPPED;
/*********/
S334:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S352;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S353;}
goto S_TRAPPED;
/*********/
S333:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:facing-north-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S352;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH__CONDITION
#define FACING_NORTH__CONDITION (1)
#endif
#ifndef FACING_NORTH__ACTION
#define FACING_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_NORTH__CONDITION))) { TRACING__ACTION; FACING_NORTH__ACTION; goto S354;}
goto S_TRAPPED;
/*********/
S332:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S355;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S356;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S357;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S358;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S359;}
goto S_TRAPPED;
/*********/
S331:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S360;}
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S361;}
goto S_TRAPPED;
/*********/
S330:
fsm_current_state = "second-leg:horizontal-search:turn-upwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S300;}
goto S_TRAPPED;
/*********/
S329:
fsm_current_state = "second-leg:horizontal-search:continue-downwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S299;}
goto S_TRAPPED;
/*********/
S328:
fsm_current_state = "second-leg:horizontal-search:lvl-below:horizontal-search-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
goto S_TRAPPED;
/*********/
S327:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACE_FOUND_LOOP__CONDITION
#define TRACE_FOUND_LOOP__CONDITION (1)
#endif
#ifndef TRACE_FOUND_LOOP__ACTION
#define TRACE_FOUND_LOOP__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACE_FOUND_LOOP__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACE_FOUND_LOOP__ACTION; goto S362;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S363;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S364;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S365;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S366;}
goto S_TRAPPED;
/*********/
S326:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S367;}
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S368;}
goto S_TRAPPED;
/*********/
S325:
fsm_current_state = "second-leg:horizontal-search:turn-downwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S299;}
goto S_TRAPPED;
/*********/
S324:
fsm_current_state = "second-leg:horizontal-search:continue-upwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S300;}
goto S_TRAPPED;
/*********/
S323:
fsm_current_state = "second-leg:horizontal-search:lvl-above:horizontal-search-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
goto S_TRAPPED;
/*********/
S368:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S87;}
goto S_TRAPPED;
/*********/
S367:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S290;}
goto S_TRAPPED;
/*********/
S366:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S369;}
goto S_TRAPPED;
/*********/
S365:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S321;}
goto S_TRAPPED;
/*********/
S364:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S370;}
goto S_TRAPPED;
/*********/
S363:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S371;}
goto S_TRAPPED;
/*********/
S362:
fsm_current_state = "second-leg:lvl-search-ascending:first-leg:horizontal-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S372;}
goto S_TRAPPED;
/*********/
S361:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S98;}
goto S_TRAPPED;
/*********/
S360:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S289;}
goto S_TRAPPED;
/*********/
S359:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S373;}
goto S_TRAPPED;
/*********/
S358:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S319;}
goto S_TRAPPED;
/*********/
S357:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S374;}
goto S_TRAPPED;
/*********/
S356:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S375;}
goto S_TRAPPED;
/*********/
S355:
fsm_current_state = "second-leg:lvl-search-descending:first-leg:horizontal-search:trace-found-loop";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_DRAW_CLOSED__CONDITION
#define TRACE_DRAW_CLOSED__CONDITION (1)
#endif
#ifndef TRACE_DRAW_CLOSED__ACTION
#define TRACE_DRAW_CLOSED__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_CLOSED__CONDITION))) { TRACE_DRAW_CLOSED__ACTION; goto S376;}
goto S_TRAPPED;
/*********/
S354:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S377;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S378;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S379;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S380;}
goto S_TRAPPED;
/*********/
S353:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S381;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S382;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S383;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S384;}
goto S_TRAPPED;
/*********/
S352:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S385;}
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S386;}
goto S_TRAPPED;
/*********/
S351:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:trace-draw-open";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S349;}
goto S_TRAPPED;
/*********/
S350:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:end-first-leg";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef SECOND_LEG__CONDITION
#define SECOND_LEG__CONDITION (1)
#endif
#ifndef SECOND_LEG__ACTION
#define SECOND_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (SECOND_LEG__CONDITION) && (FACING_WEST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; SECOND_LEG__ACTION; FACING_WEST__ACTION; goto S118;}
goto S_TRAPPED;
/*********/
S349:
fsm_current_state = "second-leg:lvl-search-ascending:first-leg:west-edge-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_INCREMENT__CONDITION
#define LVL_INCREMENT__CONDITION (1)
#endif
#ifndef LVL_INCREMENT__ACTION
#define LVL_INCREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_INCREMENT__CONDITION))) { LVL_INCREMENT__ACTION; goto S387;}
goto S_TRAPPED;
/*********/
S348:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:facing-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_NORTH_EDGE__CONDITION
#define AT_NORTH_EDGE__CONDITION (1)
#endif
#ifndef AT_NORTH_EDGE__ACTION
#define AT_NORTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_NORTH_EDGE__CONDITION))) { TRACING__ACTION; AT_NORTH_EDGE__ACTION; goto S388;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_WEST__CONDITION
#define GO_NORTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_WEST__ACTION
#define GO_NORTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_WEST__ACTION; goto S389;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_NORTH__CONDITION
#define GO_NORTH_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_NORTH__ACTION
#define GO_NORTH_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_NORTH__ACTION; goto S390;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_NORTH_TOWARD_EAST__CONDITION
#define GO_NORTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_NORTH_TOWARD_EAST__ACTION
#define GO_NORTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_NORTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_NORTH_TOWARD_EAST__ACTION; goto S391;}
goto S_TRAPPED;
/*********/
S347:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S392;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S393;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S394;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S395;}
goto S_TRAPPED;
/*********/
S346:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:trace-hit-mark";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S396;}
#ifndef TRACE_CANCELLED__CONDITION
#define TRACE_CANCELLED__CONDITION (1)
#endif
#ifndef TRACE_CANCELLED__ACTION
#define TRACE_CANCELLED__ACTION { /* no-op */ }
#endif
if (((TRACE_CANCELLED__CONDITION))) { TRACE_CANCELLED__ACTION; goto S397;}
goto S_TRAPPED;
/*********/
S345:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:trace-draw-open";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S343;}
goto S_TRAPPED;
/*********/
S344:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:end-first-leg";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef SECOND_LEG__CONDITION
#define SECOND_LEG__CONDITION (1)
#endif
#ifndef SECOND_LEG__ACTION
#define SECOND_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (SECOND_LEG__CONDITION) && (FACING_WEST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; SECOND_LEG__ACTION; FACING_WEST__ACTION; goto S129;}
goto S_TRAPPED;
/*********/
S343:
fsm_current_state = "second-leg:lvl-search-descending:first-leg:west-edge-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DECREMENT__CONDITION
#define LVL_DECREMENT__CONDITION (1)
#endif
#ifndef LVL_DECREMENT__ACTION
#define LVL_DECREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_DECREMENT__CONDITION))) { LVL_DECREMENT__ACTION; goto S398;}
goto S_TRAPPED;
/*********/
S398:
fsm_current_state = "second-leg:first-leg:west-edge-search:lvl-decrement";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S399;}
goto S_TRAPPED;
/*********/
S397:
fsm_current_state = "lvl-search-descending:west-edge-search:first-leg:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S343;}
goto S_TRAPPED;
/*********/
S396:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S343;}
goto S_TRAPPED;
/*********/
S395:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S400;}
goto S_TRAPPED;
/*********/
S394:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S340;}
goto S_TRAPPED;
/*********/
S393:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S307;}
goto S_TRAPPED;
/*********/
S392:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S341;}
goto S_TRAPPED;
/*********/
S391:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S307;}
goto S_TRAPPED;
/*********/
S390:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S339;}
goto S_TRAPPED;
/*********/
S389:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S400;}
goto S_TRAPPED;
/*********/
S388:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S341;}
goto S_TRAPPED;
/*********/
S387:
fsm_current_state = "second-leg:first-leg:west-edge-search:lvl-increment";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S401;}
goto S_TRAPPED;
/*********/
S386:
fsm_current_state = "lvl-search-ascending:west-edge-search:first-leg:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S349;}
goto S_TRAPPED;
/*********/
S385:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:trace-cancelled";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S349;}
goto S_TRAPPED;
/*********/
S384:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S402;}
goto S_TRAPPED;
/*********/
S383:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S334;}
goto S_TRAPPED;
/*********/
S382:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S305;}
goto S_TRAPPED;
/*********/
S381:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S335;}
goto S_TRAPPED;
/*********/
S380:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-north-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S305;}
goto S_TRAPPED;
/*********/
S379:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-north-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S333;}
goto S_TRAPPED;
/*********/
S378:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-north-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S402;}
goto S_TRAPPED;
/*********/
S377:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:at-north-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S335;}
goto S_TRAPPED;
/*********/
S376:
fsm_current_state = "second-leg:lvl-search-descending:first-leg:horizontal-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S403;}
goto S_TRAPPED;
/*********/
S375:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef END_FIRST_LEG__CONDITION
#define END_FIRST_LEG__CONDITION (1)
#endif
#ifndef END_FIRST_LEG__ACTION
#define END_FIRST_LEG__ACTION { /* no-op */ }
#endif
if (((END_FIRST_LEG__CONDITION))) { END_FIRST_LEG__ACTION; goto S404;}
#ifndef TRACE_DRAW_OPEN__CONDITION
#define TRACE_DRAW_OPEN__CONDITION (1)
#endif
#ifndef TRACE_DRAW_OPEN__ACTION
#define TRACE_DRAW_OPEN__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_OPEN__CONDITION))) { TRACE_DRAW_OPEN__ACTION; goto S405;}
goto S_TRAPPED;
/*********/
S374:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S406;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S407;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S408;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S409;}
goto S_TRAPPED;
/*********/
S373:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S410;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S411;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S412;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S413;}
goto S_TRAPPED;
/*********/
S372:
fsm_current_state = "second-leg:lvl-search-ascending:first-leg:horizontal-search:trace-draw-closed";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S414;}
goto S_TRAPPED;
/*********/
S371:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:trace-at-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef END_FIRST_LEG__CONDITION
#define END_FIRST_LEG__CONDITION (1)
#endif
#ifndef END_FIRST_LEG__ACTION
#define END_FIRST_LEG__ACTION { /* no-op */ }
#endif
if (((END_FIRST_LEG__CONDITION))) { END_FIRST_LEG__ACTION; goto S415;}
#ifndef TRACE_DRAW_OPEN__CONDITION
#define TRACE_DRAW_OPEN__CONDITION (1)
#endif
#ifndef TRACE_DRAW_OPEN__ACTION
#define TRACE_DRAW_OPEN__ACTION { /* no-op */ }
#endif
if (((TRACE_DRAW_OPEN__CONDITION))) { TRACE_DRAW_OPEN__ACTION; goto S416;}
goto S_TRAPPED;
/*********/
S370:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S417;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S418;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S419;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S420;}
goto S_TRAPPED;
/*********/
S369:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:facing-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_EAST_EDGE__CONDITION
#define AT_EAST_EDGE__CONDITION (1)
#endif
#ifndef AT_EAST_EDGE__ACTION
#define AT_EAST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_EAST_EDGE__CONDITION))) { TRACING__ACTION; AT_EAST_EDGE__ACTION; goto S421;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_SOUTH__CONDITION
#define GO_EAST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_SOUTH__ACTION
#define GO_EAST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_SOUTH__ACTION; goto S422;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_EAST__CONDITION
#define GO_EAST_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_EAST__ACTION
#define GO_EAST_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_EAST__ACTION; goto S423;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_EAST_TOWARD_NORTH__CONDITION
#define GO_EAST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_EAST_TOWARD_NORTH__ACTION
#define GO_EAST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_EAST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_EAST_TOWARD_NORTH__ACTION; goto S424;}
goto S_TRAPPED;
/*********/
S424:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S321;}
goto S_TRAPPED;
/*********/
S423:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S369;}
goto S_TRAPPED;
/*********/
S422:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S425;}
goto S_TRAPPED;
/*********/
S421:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S371;}
goto S_TRAPPED;
/*********/
S420:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S425;}
goto S_TRAPPED;
/*********/
S419:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S370;}
goto S_TRAPPED;
/*********/
S418:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S321;}
goto S_TRAPPED;
/*********/
S417:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S371;}
goto S_TRAPPED;
/*********/
S416:
fsm_current_state = "lvl-search-ascending:horizontal-search:first-leg:trace-draw-open";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S414;}
goto S_TRAPPED;
/*********/
S415:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:end-first-leg";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef SECOND_LEG__CONDITION
#define SECOND_LEG__CONDITION (1)
#endif
#ifndef SECOND_LEG__ACTION
#define SECOND_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (SECOND_LEG__CONDITION) && (FACING_SOUTH__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; SECOND_LEG__ACTION; FACING_SOUTH__ACTION; goto S147;}
goto S_TRAPPED;
/*********/
S414:
fsm_current_state = "second-leg:lvl-search-ascending:first-leg:horizontal-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_INCREMENT__CONDITION
#define LVL_INCREMENT__CONDITION (1)
#endif
#ifndef LVL_INCREMENT__ACTION
#define LVL_INCREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_INCREMENT__CONDITION))) { LVL_INCREMENT__ACTION; goto S426;}
goto S_TRAPPED;
/*********/
S413:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-east-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S319;}
goto S_TRAPPED;
/*********/
S412:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-east-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S373;}
goto S_TRAPPED;
/*********/
S411:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-east-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S427;}
goto S_TRAPPED;
/*********/
S410:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:at-east-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S375;}
goto S_TRAPPED;
/*********/
S409:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S427;}
goto S_TRAPPED;
/*********/
S408:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S374;}
goto S_TRAPPED;
/*********/
S407:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S319;}
goto S_TRAPPED;
/*********/
S406:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S375;}
goto S_TRAPPED;
/*********/
S405:
fsm_current_state = "lvl-search-descending:horizontal-search:first-leg:trace-draw-open";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_COMPLETE__CONDITION
#define TRACE_COMPLETE__CONDITION (1)
#endif
#ifndef TRACE_COMPLETE__ACTION
#define TRACE_COMPLETE__ACTION { /* no-op */ }
#endif
if (((TRACE_COMPLETE__CONDITION))) { TRACE_COMPLETE__ACTION; goto S403;}
goto S_TRAPPED;
/*********/
S404:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:end-first-leg";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef SECOND_LEG__CONDITION
#define SECOND_LEG__CONDITION (1)
#endif
#ifndef SECOND_LEG__ACTION
#define SECOND_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (SECOND_LEG__CONDITION) && (FACING_SOUTH__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; SECOND_LEG__ACTION; FACING_SOUTH__ACTION; goto S144;}
goto S_TRAPPED;
/*********/
S403:
fsm_current_state = "second-leg:lvl-search-descending:first-leg:horizontal-search:trace-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DECREMENT__CONDITION
#define LVL_DECREMENT__CONDITION (1)
#endif
#ifndef LVL_DECREMENT__ACTION
#define LVL_DECREMENT__ACTION { /* no-op */ }
#endif
if (((LVL_DECREMENT__CONDITION))) { LVL_DECREMENT__ACTION; goto S428;}
goto S_TRAPPED;
/*********/
S402:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S429;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S430;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S431;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S432;}
goto S_TRAPPED;
/*********/
S401:
fsm_current_state = "second-leg:first-leg:west-edge-search:lvl-search-ascend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_ASCEND_COMPLETE__CONDITION
#define LVL_ASCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_ASCEND_COMPLETE__ACTION
#define LVL_ASCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_ASCEND_COMPLETE__CONDITION))) { LVL_ASCEND_COMPLETE__ACTION; goto S433;}
#ifndef LVL_SEARCH_ASCENDING__CONDITION
#define LVL_SEARCH_ASCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCENDING__ACTION
#define LVL_SEARCH_ASCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_ASCENDING__ACTION; TRACE_START__ACTION; goto S434;}
goto S_TRAPPED;
/*********/
S400:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:facing-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_WEST_EDGE__CONDITION
#define AT_WEST_EDGE__CONDITION (1)
#endif
#ifndef AT_WEST_EDGE__ACTION
#define AT_WEST_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_WEST_EDGE__CONDITION))) { TRACING__ACTION; AT_WEST_EDGE__ACTION; goto S435;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_NORTH__CONDITION
#define GO_WEST_TOWARD_NORTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_NORTH__ACTION
#define GO_WEST_TOWARD_NORTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_NORTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_NORTH__ACTION; goto S436;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_WEST__CONDITION
#define GO_WEST_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_WEST__ACTION
#define GO_WEST_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_WEST__ACTION; goto S437;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_WEST_TOWARD_SOUTH__CONDITION
#define GO_WEST_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_WEST_TOWARD_SOUTH__ACTION
#define GO_WEST_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_WEST_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_WEST_TOWARD_SOUTH__ACTION; goto S438;}
goto S_TRAPPED;
/*********/
S399:
fsm_current_state = "second-leg:first-leg:west-edge-search:lvl-search-descend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DESCEND_COMPLETE__CONDITION
#define LVL_DESCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_DESCEND_COMPLETE__ACTION
#define LVL_DESCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_DESCEND_COMPLETE__CONDITION))) { LVL_DESCEND_COMPLETE__ACTION; goto S439;}
#ifndef LVL_SEARCH_DESCENDING__CONDITION
#define LVL_SEARCH_DESCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCENDING__ACTION
#define LVL_SEARCH_DESCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_DESCENDING__ACTION; TRACE_START__ACTION; goto S440;}
goto S_TRAPPED;
/*********/
S440:
fsm_current_state = "second-leg:first-leg:west-edge-search:lvl-search-descending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_EAST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_EAST__ACTION; goto S307;}
goto S_TRAPPED;
/*********/
S439:
fsm_current_state = "second-leg:first-leg:west-edge-search:lvl-descend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_INIT__CONDITION
#define HORIZONTAL_SEARCH_INIT__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_INIT__ACTION
#define HORIZONTAL_SEARCH_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_INIT__CONDITION))) { HORIZONTAL_SEARCH_INIT__ACTION; goto S441;}
goto S_TRAPPED;
/*********/
S438:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S340;}
goto S_TRAPPED;
/*********/
S437:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S400;}
goto S_TRAPPED;
/*********/
S436:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S339;}
goto S_TRAPPED;
/*********/
S435:
fsm_current_state = "second-leg:lvl-search-descending:west-edge-search:first-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S341;}
goto S_TRAPPED;
/*********/
S434:
fsm_current_state = "second-leg:first-leg:west-edge-search:lvl-search-ascending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef WEST_EDGE_SEARCH__CONDITION
#define WEST_EDGE_SEARCH__CONDITION (1)
#endif
#ifndef WEST_EDGE_SEARCH__ACTION
#define WEST_EDGE_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((WEST_EDGE_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_EAST__CONDITION))) { WEST_EDGE_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_EAST__ACTION; goto S305;}
goto S_TRAPPED;
/*********/
S433:
fsm_current_state = "second-leg:first-leg:west-edge-search:lvl-ascend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_INIT__CONDITION
#define HORIZONTAL_SEARCH_INIT__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_INIT__ACTION
#define HORIZONTAL_SEARCH_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_INIT__CONDITION))) { HORIZONTAL_SEARCH_INIT__ACTION; goto S441;}
goto S_TRAPPED;
/*********/
S432:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-west-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S334;}
goto S_TRAPPED;
/*********/
S431:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-west-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S402;}
goto S_TRAPPED;
/*********/
S430:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:go-west-toward-north";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_NORTH_TESTING_MARKS__CONDITION))) { FACING_NORTH_TESTING_MARKS__ACTION; goto S333;}
goto S_TRAPPED;
/*********/
S429:
fsm_current_state = "second-leg:lvl-search-ascending:west-edge-search:first-leg:tracing:at-west-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S335;}
goto S_TRAPPED;
/*********/
S428:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-decrement";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S442;}
goto S_TRAPPED;
/*********/
S427:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S331;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S443;}
goto S_TRAPPED;
/*********/
S426:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-increment";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S444;}
goto S_TRAPPED;
/*********/
S425:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:facing-south-testing-marks";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_HIT_MARK__CONDITION
#define TRACE_HIT_MARK__CONDITION (1)
#endif
#ifndef TRACE_HIT_MARK__ACTION
#define TRACE_HIT_MARK__ACTION { /* no-op */ }
#endif
if (((TRACE_HIT_MARK__CONDITION))) { TRACE_HIT_MARK__ACTION; goto S326;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FACING_SOUTH__CONDITION
#define FACING_SOUTH__CONDITION (1)
#endif
#ifndef FACING_SOUTH__ACTION
#define FACING_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (FACING_SOUTH__CONDITION))) { TRACING__ACTION; FACING_SOUTH__ACTION; goto S445;}
goto S_TRAPPED;
/*********/
S445:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S446;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S447;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S448;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S449;}
goto S_TRAPPED;
/*********/
S444:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-search-ascend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_ASCEND_COMPLETE__CONDITION
#define LVL_ASCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_ASCEND_COMPLETE__ACTION
#define LVL_ASCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_ASCEND_COMPLETE__CONDITION))) { LVL_ASCEND_COMPLETE__ACTION; goto S450;}
#ifndef LVL_SEARCH_ASCENDING__CONDITION
#define LVL_SEARCH_ASCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCENDING__ACTION
#define LVL_SEARCH_ASCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_ASCENDING__ACTION; TRACE_START__ACTION; goto S451;}
goto S_TRAPPED;
/*********/
S443:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:facing-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef AT_SOUTH_EDGE__CONDITION
#define AT_SOUTH_EDGE__CONDITION (1)
#endif
#ifndef AT_SOUTH_EDGE__ACTION
#define AT_SOUTH_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (AT_SOUTH_EDGE__CONDITION))) { TRACING__ACTION; AT_SOUTH_EDGE__ACTION; goto S452;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_EAST__CONDITION
#define GO_SOUTH_TOWARD_EAST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_EAST__ACTION
#define GO_SOUTH_TOWARD_EAST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_EAST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_EAST__ACTION; goto S453;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__CONDITION
#define GO_SOUTH_TOWARD_SOUTH__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_SOUTH__ACTION
#define GO_SOUTH_TOWARD_SOUTH__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_SOUTH__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_SOUTH__ACTION; goto S454;}
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef GO_SOUTH_TOWARD_WEST__CONDITION
#define GO_SOUTH_TOWARD_WEST__CONDITION (1)
#endif
#ifndef GO_SOUTH_TOWARD_WEST__ACTION
#define GO_SOUTH_TOWARD_WEST__ACTION { /* no-op */ }
#endif
if (((TRACING__CONDITION) && (GO_SOUTH_TOWARD_WEST__CONDITION))) { TRACING__ACTION; GO_SOUTH_TOWARD_WEST__ACTION; goto S455;}
goto S_TRAPPED;
/*********/
S442:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-search-descend-enter";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_DESCEND_COMPLETE__CONDITION
#define LVL_DESCEND_COMPLETE__CONDITION (1)
#endif
#ifndef LVL_DESCEND_COMPLETE__ACTION
#define LVL_DESCEND_COMPLETE__ACTION { /* no-op */ }
#endif
if (((LVL_DESCEND_COMPLETE__CONDITION))) { LVL_DESCEND_COMPLETE__ACTION; goto S456;}
#ifndef LVL_SEARCH_DESCENDING__CONDITION
#define LVL_SEARCH_DESCENDING__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCENDING__ACTION
#define LVL_SEARCH_DESCENDING__ACTION { /* no-op */ }
#endif
#ifndef TRACE_START__CONDITION
#define TRACE_START__CONDITION (1)
#endif
#ifndef TRACE_START__ACTION
#define TRACE_START__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCENDING__CONDITION) && (TRACE_START__CONDITION))) { LVL_SEARCH_DESCENDING__ACTION; TRACE_START__ACTION; goto S457;}
goto S_TRAPPED;
/*********/
S441:
fsm_current_state = "second-leg:first-leg:horizontal-search-init";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_SEEK_INIT__CONDITION
#define LVL_SEEK_INIT__CONDITION (1)
#endif
#ifndef LVL_SEEK_INIT__ACTION
#define LVL_SEEK_INIT__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_SEEK_INIT__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_SEEK_INIT__ACTION; goto S458;}
goto S_TRAPPED;
/*********/
S458:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-seek-init";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEEK_FROM_TOP__CONDITION
#define LVL_SEEK_FROM_TOP__CONDITION (1)
#endif
#ifndef LVL_SEEK_FROM_TOP__ACTION
#define LVL_SEEK_FROM_TOP__ACTION { /* no-op */ }
#endif
if (((LVL_SEEK_FROM_TOP__CONDITION))) { LVL_SEEK_FROM_TOP__ACTION; goto S459;}
#ifndef LVL_SEEK_FROM_BOTTOM__CONDITION
#define LVL_SEEK_FROM_BOTTOM__CONDITION (1)
#endif
#ifndef LVL_SEEK_FROM_BOTTOM__ACTION
#define LVL_SEEK_FROM_BOTTOM__ACTION { /* no-op */ }
#endif
if (((LVL_SEEK_FROM_BOTTOM__CONDITION))) { LVL_SEEK_FROM_BOTTOM__ACTION; goto S460;}
goto S_TRAPPED;
/*********/
S457:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-search-descending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_NORTH_TESTING_MARKS__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_NORTH_TESTING_MARKS__ACTION; goto S319;}
goto S_TRAPPED;
/*********/
S456:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-descend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_BELOW__CONDITION
#define LVL_BELOW__CONDITION (1)
#endif
#ifndef LVL_BELOW__ACTION
#define LVL_BELOW__ACTION { /* no-op */ }
#endif
#ifndef COLUMN_INCREASE__CONDITION
#define COLUMN_INCREASE__CONDITION (1)
#endif
#ifndef COLUMN_INCREASE__ACTION
#define COLUMN_INCREASE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_BELOW__CONDITION) && (COLUMN_INCREASE__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_BELOW__ACTION; COLUMN_INCREASE__ACTION; goto S461;}
goto S_TRAPPED;
/*********/
S455:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S374;}
goto S_TRAPPED;
/*********/
S454:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S427;}
goto S_TRAPPED;
/*********/
S453:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S373;}
goto S_TRAPPED;
/*********/
S452:
fsm_current_state = "second-leg:lvl-search-descending:horizontal-search:first-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S375;}
goto S_TRAPPED;
/*********/
S451:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-search-ascending:trace-start";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef TRACING__CONDITION
#define TRACING__CONDITION (1)
#endif
#ifndef TRACING__ACTION
#define TRACING__ACTION { /* no-op */ }
#endif
#ifndef FIRST_LEG__CONDITION
#define FIRST_LEG__CONDITION (1)
#endif
#ifndef FIRST_LEG__ACTION
#define FIRST_LEG__ACTION { /* no-op */ }
#endif
#ifndef FACING_NORTH_TESTING_MARKS__CONDITION
#define FACING_NORTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_NORTH_TESTING_MARKS__ACTION
#define FACING_NORTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (TRACING__CONDITION) && (FIRST_LEG__CONDITION) && (FACING_NORTH_TESTING_MARKS__CONDITION))) { HORIZONTAL_SEARCH__ACTION; TRACING__ACTION; FIRST_LEG__ACTION; FACING_NORTH_TESTING_MARKS__ACTION; goto S321;}
goto S_TRAPPED;
/*********/
S450:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-ascend-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH__CONDITION
#define HORIZONTAL_SEARCH__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH__ACTION
#define HORIZONTAL_SEARCH__ACTION { /* no-op */ }
#endif
#ifndef LVL_ABOVE__CONDITION
#define LVL_ABOVE__CONDITION (1)
#endif
#ifndef LVL_ABOVE__ACTION
#define LVL_ABOVE__ACTION { /* no-op */ }
#endif
#ifndef COLUMN_INCREASE__CONDITION
#define COLUMN_INCREASE__CONDITION (1)
#endif
#ifndef COLUMN_INCREASE__ACTION
#define COLUMN_INCREASE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH__CONDITION) && (LVL_ABOVE__CONDITION) && (COLUMN_INCREASE__CONDITION))) { HORIZONTAL_SEARCH__ACTION; LVL_ABOVE__ACTION; COLUMN_INCREASE__ACTION; goto S462;}
goto S_TRAPPED;
/*********/
S449:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-south-toward-west";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_WEST__CONDITION
#define FACING_WEST__CONDITION (1)
#endif
#ifndef FACING_WEST__ACTION
#define FACING_WEST__ACTION { /* no-op */ }
#endif
if (((FACING_WEST__CONDITION))) { FACING_WEST__ACTION; goto S370;}
goto S_TRAPPED;
/*********/
S448:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-south-toward-south";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__CONDITION
#define FACING_SOUTH_TESTING_MARKS__CONDITION (1)
#endif
#ifndef FACING_SOUTH_TESTING_MARKS__ACTION
#define FACING_SOUTH_TESTING_MARKS__ACTION { /* no-op */ }
#endif
if (((FACING_SOUTH_TESTING_MARKS__CONDITION))) { FACING_SOUTH_TESTING_MARKS__ACTION; goto S425;}
goto S_TRAPPED;
/*********/
S447:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:go-south-toward-east";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef FACING_EAST__CONDITION
#define FACING_EAST__CONDITION (1)
#endif
#ifndef FACING_EAST__ACTION
#define FACING_EAST__ACTION { /* no-op */ }
#endif
if (((FACING_EAST__CONDITION))) { FACING_EAST__ACTION; goto S369;}
goto S_TRAPPED;
/*********/
S446:
fsm_current_state = "second-leg:lvl-search-ascending:horizontal-search:first-leg:tracing:at-south-edge";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef TRACE_AT_EDGE__CONDITION
#define TRACE_AT_EDGE__CONDITION (1)
#endif
#ifndef TRACE_AT_EDGE__ACTION
#define TRACE_AT_EDGE__ACTION { /* no-op */ }
#endif
if (((TRACE_AT_EDGE__CONDITION))) { TRACE_AT_EDGE__ACTION; goto S371;}
goto S_TRAPPED;
/*********/
S462:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-above:column-increase";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__CONDITION
#define HORIZONTAL_SEARCH_COMPLETE__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__ACTION
#define HORIZONTAL_SEARCH_COMPLETE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_COMPLETE__CONDITION))) { HORIZONTAL_SEARCH_COMPLETE__ACTION; goto S463;}
#ifndef CONTINUE_UPWARDS__CONDITION
#define CONTINUE_UPWARDS__CONDITION (1)
#endif
#ifndef CONTINUE_UPWARDS__ACTION
#define CONTINUE_UPWARDS__ACTION { /* no-op */ }
#endif
if (((CONTINUE_UPWARDS__CONDITION))) { CONTINUE_UPWARDS__ACTION; goto S464;}
#ifndef TURN_DOWNWARDS__CONDITION
#define TURN_DOWNWARDS__CONDITION (1)
#endif
#ifndef TURN_DOWNWARDS__ACTION
#define TURN_DOWNWARDS__ACTION { /* no-op */ }
#endif
if (((TURN_DOWNWARDS__CONDITION))) { TURN_DOWNWARDS__ACTION; goto S465;}
goto S_TRAPPED;
/*********/
S461:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-below:column-increase";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__CONDITION
#define HORIZONTAL_SEARCH_COMPLETE__CONDITION (1)
#endif
#ifndef HORIZONTAL_SEARCH_COMPLETE__ACTION
#define HORIZONTAL_SEARCH_COMPLETE__ACTION { /* no-op */ }
#endif
if (((HORIZONTAL_SEARCH_COMPLETE__CONDITION))) { HORIZONTAL_SEARCH_COMPLETE__ACTION; goto S466;}
#ifndef CONTINUE_DOWNWARDS__CONDITION
#define CONTINUE_DOWNWARDS__CONDITION (1)
#endif
#ifndef CONTINUE_DOWNWARDS__ACTION
#define CONTINUE_DOWNWARDS__ACTION { /* no-op */ }
#endif
if (((CONTINUE_DOWNWARDS__CONDITION))) { CONTINUE_DOWNWARDS__ACTION; goto S467;}
#ifndef TURN_UPWARDS__CONDITION
#define TURN_UPWARDS__CONDITION (1)
#endif
#ifndef TURN_UPWARDS__ACTION
#define TURN_UPWARDS__ACTION { /* no-op */ }
#endif
if (((TURN_UPWARDS__CONDITION))) { TURN_UPWARDS__ACTION; goto S468;}
goto S_TRAPPED;
/*********/
S460:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-seek-from-bottom";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S444;}
goto S_TRAPPED;
/*********/
S459:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-seek-from-top";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S442;}
goto S_TRAPPED;
/*********/
S468:
fsm_current_state = "second-leg:first-leg:horizontal-search:turn-upwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S444;}
goto S_TRAPPED;
/*********/
S467:
fsm_current_state = "second-leg:first-leg:horizontal-search:continue-downwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S442;}
goto S_TRAPPED;
/*********/
S466:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-below:horizontal-search-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
goto S_TRAPPED;
/*********/
S465:
fsm_current_state = "second-leg:first-leg:horizontal-search:turn-downwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__CONDITION
#define LVL_SEARCH_DESCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_DESCEND_ENTER__ACTION
#define LVL_SEARCH_DESCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_DESCEND_ENTER__CONDITION))) { LVL_SEARCH_DESCEND_ENTER__ACTION; goto S442;}
goto S_TRAPPED;
/*********/
S464:
fsm_current_state = "second-leg:first-leg:horizontal-search:continue-upwards";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__CONDITION
#define LVL_SEARCH_ASCEND_ENTER__CONDITION (1)
#endif
#ifndef LVL_SEARCH_ASCEND_ENTER__ACTION
#define LVL_SEARCH_ASCEND_ENTER__ACTION { /* no-op */ }
#endif
if (((LVL_SEARCH_ASCEND_ENTER__CONDITION))) { LVL_SEARCH_ASCEND_ENTER__ACTION; goto S444;}
goto S_TRAPPED;
/*********/
S463:
fsm_current_state = "second-leg:first-leg:horizontal-search:lvl-above:horizontal-search-complete";
#ifdef FSM_VERBOSE
fprintf(stderr, "%s\n", fsm_current_state);
#endif
goto S_TRAPPED;
/*********/

S_TRAPPED:
#ifndef FSM_TRAPPED
   fprintf(stderr, "fsm: trapped in state \"%s\"\n", fsm_current_state);
   exit(1);
#else
FSM_TRAPPED
#endif


contour_done:
  row++;
  RESULT_FREE;
  return FALSE;

}
